@@ -33,6 +33,7 @@ enum kernel_interface {
   USE_WIN2K = 5,
   USE_FUNK  = 6,
   USE_MASTKLIPS = 7,
+  USE_BSDKAME = 8,
 };
 
 /* RFC 3706 Dead Peer Detection */
@@ -312,8 +313,8 @@ enum phase1_role {
 #define IS_CHILD_SA_ESTABLISHED(st) (((st->st_state) == STATE_PARENT_I3 || (st->st_state) == STATE_PARENT_R2) && (st->st_childsa != NULL))
 
 
-#define IS_PARENT_SA(st) ((st)->st_clonedfrom == SOS_NOBODY)
 #define IS_CHILD_SA(st)  ((st)->st_clonedfrom != SOS_NOBODY)
+#define IS_PARENT_SA(st) (!IS_CHILD_SA(st))
 
 
 /* kind of struct connection
@@ -446,6 +447,10 @@ enum pluto_policy {
 	POLICY_IKEV2_ALLOW   = LELEM(25), /* accept IKEv2?   0x0200 0000 */
 	POLICY_IKEV2_PROPOSE = LELEM(26), /* propose IKEv2?  0x0400 0000 */
 	POLICY_IKEV2_MASK = POLICY_IKEV1_DISABLE|POLICY_IKEV2_ALLOW|POLICY_IKEV2_PROPOSE,
+	POLICY_MODECFGDNS1  = LELEM(27),   /* should we offer a DNS server IP */
+	POLICY_MODECFGDNS2  = LELEM(28),   /* should we offer another DNS server IP */
+	POLICY_MODECFGWINS1 = LELEM(29),   /* should we offer a WINS server IP */
+	POLICY_MODECFGWINS2 = LELEM(30),   /* should we offer another WINS server IP */
 };
 
 /* Any IPsec policy?  If not, a connection description
