@@ -63,9 +63,7 @@ static void default_values (struct starter_config *cfg)
 	cfg->setup.options[KBF_UNIQUEIDS]= FALSE;
 	cfg->conn_default.options[KBF_TYPE] = KS_TUNNEL;
 
-	cfg->conn_default.policy =
-	    POLICY_RSASIG|POLICY_TUNNEL|POLICY_ENCRYPT|POLICY_PFS;
-	
+	cfg->conn_default.policy = POLICY_RSASIG|POLICY_TUNNEL|POLICY_ENCRYPT|POLICY_PFS;
 	cfg->conn_default.policy |= POLICY_IKEV2_ALLOW;
 
 	cfg->conn_default.options[KBF_IKELIFETIME] = OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT;
@@ -961,8 +959,6 @@ static int load_conn (struct starter_config *cfg
 		    conn->options[KBF_AUTHBY]);
 #endif
     }
-
-
     
     KW_POLICY_FLAG(KBF_REKEY, POLICY_DONT_REKEY);
 
@@ -970,6 +966,8 @@ static int load_conn (struct starter_config *cfg
 
     KW_POLICY_FLAG(KBF_MODECONFIGPULL, POLICY_MODECFG_PULL);
 
+    KW_POLICY_FLAG(KBF_OVERLAPIP, POLICY_OVERLAPIP);
+
     if(conn->strings_set[KSF_ESP]) {
 	conn->esp = xstrdup(conn->strings[KSF_ESP]);
     }
