@@ -99,9 +99,6 @@ typedef unsigned short int IPsecRefTableUnusedCount;
 #define IPSEC_SA_REF_SUBTABLE_NUM_ENTRIES (1 << IPSEC_SA_REF_SUBTABLE_IDX_WIDTH)
 
 #ifdef CONFIG_NETFILTER
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
-#define		nfmark  mark
-#endif
 #define IPSEC_SA_REF_HOST_FIELD(x) ((struct sk_buff*)(x))->nfmark
 #define IPSEC_SA_REF_HOST_FIELD_TYPE typeof(IPSEC_SA_REF_HOST_FIELD(NULL))
 #else /* CONFIG_NETFILTER */
@@ -136,8 +133,6 @@ struct ipsec_sa
 	struct ipsec_sa	*ips_next;	 	/* pointer to next xform */
 
 	struct ipsec_sa	*ips_hnext;		/* next in hash chain */
-	struct ipsec_sa	*ips_inext;	 	/* pointer to next xform */
-	struct ipsec_sa	*ips_onext;	 	/* pointer to prev xform */
 
 	struct ifnet	*ips_rcvif;	 	/* related rcv encap interface */
 
@@ -223,7 +218,6 @@ struct ipsec_sa
 #endif
 	struct ipsec_alg_enc *ips_alg_enc;
 	struct ipsec_alg_auth *ips_alg_auth;
-//IPsecSAref_t	ips_ref_rel;
 };
 
 struct IPsecSArefSubTable
@@ -260,10 +254,11 @@ extern int ipsec_sa_add(struct ipsec_sa *ips);
 extern void ipsec_sa_rm(struct ipsec_sa *ips);
 extern int ipsec_sadb_cleanup(__u8 proto);
 extern int ipsec_sadb_free(void);
+extern int ipsec_sa_wipe(struct ipsec_sa *ips);
 extern int ipsec_sa_intern(struct ipsec_sa *ips);
-extern void ipsec_sa_untern(struct ipsec_sa *ips);
 extern struct ipsec_sa *ipsec_sa_getbyref(IPsecSAref_t ref);
 
+extern void ipsec_sa_untern(struct ipsec_sa *ips);
 #endif /* __KERNEL__ */
 
 enum ipsec_direction {
