@@ -2,6 +2,7 @@
  * IPSEC tunneling code
  * Copyright (C) 1996, 1997  John Ioannidis.
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003  Richard Guy Briggs.
+ * Copyright (C) 2006        Michael Richardson <mcr@xelerance.com>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -13,7 +14,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_tunnel.h,v 1.33 2005/06/04 16:06:05 mcr Exp $
  */
 
 
@@ -36,7 +36,7 @@
 
 struct ipsectunnelconf
 {
-	__u32	cf_cmd;
+	uint32_t	cf_cmd;
 	union
 	{
 		char 	cfu_name[12];
@@ -60,9 +60,14 @@ struct ipsecpriv
 	struct sk_buff_head sendq;
 	struct net_device *dev;
 	struct wait_queue *wait_queue;
+	int  vifnum;
 	char locked;
 	int  (*hard_start_xmit) (struct sk_buff *skb,
 		struct net_device *dev);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	struct header_ops *header_ops;
+#else
+
 	int  (*hard_header) (struct sk_buff *skb,
 		struct net_device *dev,
 		unsigned short type,
@@ -75,12 +80,13 @@ struct ipsecpriv
 	int  (*rebuild_header)(void *buff, struct net_device *dev,
 			unsigned long raddr, struct sk_buff *skb);
 #endif /* NET_21 */
-	int  (*set_mac_address)(struct net_device *dev, void *addr);
 #ifndef NET_21
 	void (*header_cache_bind)(struct hh_cache **hhp, struct net_device *dev,
 				 unsigned short htype, __u32 daddr);
 #endif /* !NET_21 */
 	void (*header_cache_update)(struct hh_cache *hh, struct net_device *dev, unsigned char *  haddr);
+#endif
+	int  (*set_mac_address)(struct net_device *dev, void *addr);
 	struct net_device_stats *(*get_stats)(struct net_device *dev);
 	struct net_device_stats mystats;
 	int mtu;	/* What is the desired MTU? */
@@ -88,7 +94,8 @@ struct ipsecpriv
 
 extern char ipsec_tunnel_c_version[];
 
-extern struct net_device *ipsecdevices[IPSEC_NUM_IF];
+extern struct net_device *ipsecdevices[IPSEC_NUM_IFMAX];
+extern int ipsecdevices_max;
 
 int ipsec_tunnel_init_devices(void);
 
@@ -116,9 +123,15 @@ extern int sysctl_ipsec_debug_verbose;
 #define DB_TN_ENCAP     0x0200
 #endif /* CONFIG_KLIPS_DEBUG */
 
-// manage ipsec xmit state objects
+extern int ipsec_tunnel_deletenum(int vifnum);
+extern int ipsec_tunnel_createnum(int vifnum);
+extern struct net_device *ipsec_tunnel_get_device(int vifnum);
+
+
+/* manage ipsec xmit state objects */
 extern int ipsec_xmit_state_cache_init (void);
 extern void ipsec_xmit_state_cache_cleanup (void);
+
 /*
  * $Log: ipsec_tunnel.h,v $
  * Revision 1.33  2005/06/04 16:06:05  mcr
