@@ -23,8 +23,8 @@
 #else
 #include <sys/types.h>
 #endif
-#include "crypto/aes_cbc.h"
-#include "crypto/cbc_generic.h"
+#include "klips-crypto/aes_cbc.h"
+#include "klips-crypto/cbc_generic.h"
 
 /* returns bool success */
 int AES_set_key(aes_context *aes_ctx, const u_int8_t *key, int keysize) {
@@ -36,7 +36,7 @@ CBC_IMPL_BLK16(AES_cbc_encrypt, aes_context, u_int8_t *, aes_encrypt, aes_decryp
 
 /*
  * $Log: aes_cbc.c,v $
- * Revision 1.2  2004-07-10 07:48:40  mcr
+ * Revision 1.2  2004/07/10 07:48:40  mcr
  * Moved from linux/crypto/ciphers/aes/aes_cbc.c,v
  *
  * Revision 1.1  2004/04/06 02:48:12  mcr
