@@ -18,12 +18,13 @@
  * for more details.
  *
  */
-#define __NO_VERSION__
-
-#if defined (MODULE)
-#include <linux/module.h>
+#ifndef AUTOCONF_INCLUDED
+#include <linux/config.h>
 #endif
+#include <linux/version.h>
 
+#define __NO_VERSION__
+#include <linux/module.h>
 #include <linux/kernel.h> /* printk() */
 
 #include <linux/netdevice.h>   /* struct device, and other headers */
@@ -69,8 +70,8 @@
 #include "openswan/ipsec_alg.h"
 #include "openswan/ipsec_proto.h"
 
-#if SADB_EALG_MAX < 255
-#warning Compiling with limited ESP support ( SADB_EALG_MAX < 256 )
+#if K_SADB_EALG_MAX < 255
+#warning Compiling with limited ESP support ( K_SADB_EALG_MAX < 256 )
 #endif
 
 static rwlock_t ipsec_alg_lock = RW_LOCK_UNLOCKED;
@@ -485,9 +486,9 @@ out:
 static int check_auth(struct ipsec_alg_auth *ixt)
 {
 	int ret=-EINVAL;
-	if (ixt->ixt_common.ixt_support.ias_id==0 || ixt->ixt_common.ixt_support.ias_id > SADB_AALG_MAX)
-		barf_out("invalid alg_id=%d > %d (SADB_AALG_MAX)\n",
-			 ixt->ixt_common.ixt_support.ias_id, SADB_AALG_MAX);
+	if (ixt->ixt_common.ixt_support.ias_id==0 || ixt->ixt_common.ixt_support.ias_id > K_SADB_AALG_MAX)
+		barf_out("invalid alg_id=%d > %d (K_SADB_AALG_MAX)\n",
+			 ixt->ixt_common.ixt_support.ias_id, K_SADB_AALG_MAX);
 
 	if (ixt->ixt_common.ixt_blocksize==0
 	    || ixt->ixt_common.ixt_blocksize%2)
@@ -567,12 +568,12 @@ int register_ipsec_alg(struct ipsec_alg *ixt)
 
 
 	ret = pfkey_list_insert_supported((struct ipsec_alg_supported *)&ixt->ixt_support
-					  , &(pfkey_supported_list[SADB_SATYPE_ESP]));
+					  , &(pfkey_supported_list[K_SADB_SATYPE_ESP]));
 
 	if (ret==0) {
 		ixt->ixt_state |= IPSEC_ALG_ST_SUPP;
 		/*	send register event to userspace	*/
-		pfkey_register_reply(SADB_SATYPE_ESP, NULL);
+		pfkey_register_reply(K_SADB_SATYPE_ESP, NULL);
 	} else
 		printk(KERN_ERR "pfkey_list_insert_supported returned %d. "
 				"Loading anyway.\n", ret);
@@ -601,10 +602,10 @@ int unregister_ipsec_alg(struct ipsec_alg *ixt) {
 	if (ixt->ixt_state&IPSEC_ALG_ST_SUPP) {
 		ixt->ixt_state &= ~IPSEC_ALG_ST_SUPP;
 		pfkey_list_remove_supported((struct ipsec_alg_supported *)&ixt->ixt_support
-					    , &(pfkey_supported_list[SADB_SATYPE_ESP]));
+					    , &(pfkey_supported_list[K_SADB_SATYPE_ESP]));
 
 		/*	send register event to userspace	*/
-		pfkey_register_reply(SADB_SATYPE_ESP, NULL);
+		pfkey_register_reply(K_SADB_SATYPE_ESP, NULL);
 	}
 
 out:
@@ -826,7 +827,7 @@ int ipsec_alg_init(void) {
 	KLIPS_PRINT(1, "klips_info:ipsec_alg_init: "
 			"KLIPS alg v=%d.%d.%d-%d (EALG_MAX=%d, AALG_MAX=%d)\n",
 			IPSEC_ALG_VERSION_QUAD(IPSEC_ALG_VERSION),
-			SADB_EALG_MAX, SADB_AALG_MAX);
+			K_SADB_EALG_MAX, K_SADB_AALG_MAX);
 	/*	Initialize tables */
 	write_lock_bh(&ipsec_alg_lock);
 	ipsec_alg_hash_init();
