@@ -36,11 +36,14 @@
  *	special case: ipsec core modular with this static algo inside:
  *	must avoid MODULE magic for this file
  */
-#if defined(CONFIG_KLIPS_MODULE) && defined(CONFIG_KLIPS_ENC_CRYPTOAPI)
+#if CONFIG_KLIPS_MODULE && CONFIG_KLIPS_ENC_CRYPTOAPI
 #undef MODULE
 #endif
 
 #include <linux/module.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+# include <linux/moduleparam.h>
+#endif
 #include <linux/init.h>
 
 #include <linux/kernel.h> /* printk() */
@@ -97,25 +100,15 @@ IPSEC_ALG_MODULE_INIT_STATIC( ipsec_cryptoapi_init )
 #define DIGESTNAME_SHA1		"sha1"
 
 MODULE_AUTHOR("Juanjo Ciarlante, Harpo MAxx, Luciano Ruete");
-static int debug_crypto=1;
-static int test_crypto=1;
+static int debug_crypto=0;
+static int test_crypto=0;
 static int excl_crypto=0;
-
 static int noauto = 0;
+module_parm(debug_crypto,int,0666);
+module_parm(test_crypto,int,0666);
+module_parm(excl_crypto,int,0666);
+module_parm(noauto,int,0666);
 
-#ifdef module_param
-module_param(debug_crypto,int,0600)
-module_param(test_crypto,int,0600)
-module_param(excl_crypto,int,0600)
-
-module_param(noauto,int,0600)
-#else
-MODULE_PARM(debug_crypto, "i");
-MODULE_PARM(test_crypto, "i");
-MODULE_PARM(excl_crypto, "i");
-
-MODULE_PARM(noauto,"i");
-#endif
 MODULE_PARM_DESC(noauto, "Dont try all known algos, just setup enabled ones");
 
 #ifdef CONFIG_KLIPS_ENC_1DES
@@ -129,27 +122,15 @@ static int serpent[] = {-1, -1};
 static int twofish[] = {-1, -1};
 
 #ifdef CONFIG_KLIPS_ENC_1DES
-#ifdef module_param
-module_param_array(des_ede1,int,NULL,0)
-#else
-MODULE_PARM(des_ede1,"1-2i");
-#endif
-#endif
-#ifdef module_param
-module_param_array(des_ede3,int,NULL,0)
-module_param_array(aes,int,NULL,0)
-module_param_array(blowfish,int,NULL,0)
-module_param_array(cast,int,NULL,0)
-module_param_array(serpent,int,NULL,0)
-module_param_array(twofish,int,NULL,0)
-#else
-MODULE_PARM(des_ede3,"1-2i");
-MODULE_PARM(aes,"1-2i");
-MODULE_PARM(blowfish,"1-2i");
-MODULE_PARM(cast,"1-2i");
-MODULE_PARM(serpent,"1-2i");
-MODULE_PARM(twofish,"1-2i");
+module_param_array(des_ede1,int,NULL,0444);
 #endif
+module_param_array(des_ede3,int,NULL,0444);
+module_param_array(aes,int,NULL,0444);
+module_param_array(blowfish,int,NULL,0444);
+module_param_array(cast,int,NULL,0444);
+module_param_array(serpent,int,NULL,0444);
+module_param_array(twofish,int,NULL,0444);
+
 MODULE_PARM_DESC(des_ede1, "0: disable | 1: force_enable | min,max: dontuse");
 MODULE_PARM_DESC(des_ede3, "0: disable | 1: force_enable | min,max: dontuse");
 MODULE_PARM_DESC(aes, "0: disable | 1: force_enable | min,max: keybitlens");
@@ -365,14 +346,7 @@ setup_cipher_list (struct ipsec_alg_capi_cipher* clist)
 					, cptr->parm[0]
 					, cptr->parm[1]);
 			continue;
-		} else 
-			if (debug_crypto>0)
-				printk(KERN_INFO "setup_cipher_list(): going to init ciphername=%s: noauto=%d parm[0]=%d parm[1]=%d\n",
-				, cptr->ciphername
-                                        , noauto
-                                        , cptr->parm[0]
-                                        , cptr->parm[1]);
-
+		}
 		/* 
 		 * 	use a local ci to avoid touching cptr->ci,
 		 * 	if register ipsec_alg success then bind cipher
