@@ -24,6 +24,9 @@
 #include <linux/version.h>
 #define __NO_VERSION__
 #include <linux/module.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) 
+#include <linux/moduleparam.h> 
+#endif 
 #include <linux/kernel.h> /* printk() */
 #include <linux/ip.h>          /* struct iphdr */
 
@@ -106,6 +109,20 @@ struct ipsec_birth_reply ipsec_ipv6_birth_packet;
 #ifdef CONFIG_KLIPS_DEBUG
 int debug_esp = 0;
 int debug_ah = 0;
+int sysctl_ipsec_inbound_policy_check = 1;
+int debug_tunnel = 0;
+int debug_xmit = 0;
+int debug_xform = 0;
+int debug_eroute = 0;
+int debug_spi = 0;
+int debug_radij = 0;
+int debug_pfkey = 0;
+int debug_rcv = 0;
+int debug_netlink = 0;
+int sysctl_ipsec_debug_verbose = 0;
+int sysctl_ipsec_debug_ipcomp =0;
+int sysctl_ipsec_icmp = 0;
+int sysctl_ipsec_tos = 0;
 #endif /* CONFIG_KLIPS_DEBUG */
 
 #define DECREMENT_UNSIGNED(X, amount) ((amount < (X)) ? (X)-amount : 0)
@@ -177,7 +194,7 @@ ipsec_spi_get_info(char *buffer,
 		for (sa_p = ipsec_sadb_hash[i];
 		     sa_p;
 		     sa_p = sa_p->ips_hnext) {
-			atomic_inc(&sa_p->ips_refcount);
+			ipsec_sa_get(sa_p);
 			sa_len = satot(&sa_p->ips_said, 'x', sa, sizeof(sa));
 			len += ipsec_snprintf(buffer+len, length-len, "%s ",
 				       sa_len ? sa : " (error)");
@@ -376,11 +393,20 @@ ipsec_spi_get_info(char *buffer,
 			len += ipsec_snprintf(buffer + len, length-len, " natencap=na");
 #endif /* CONFIG_IPSEC_NAT_TRAVERSAL */
 				
+			/* we decrement by one, because this SA has been referenced in order to dump this info */
 			len += ipsec_snprintf(buffer + len,length-len, " refcount=%d",
-				       atomic_read(&sa_p->ips_refcount));
+				       atomic_read(&sa_p->ips_refcount)-1);
 
 			len += ipsec_snprintf(buffer+len, length-len, " ref=%d",
 				       sa_p->ips_ref);
+			len += ipsec_snprintf(buffer+len, length-len, " refhim=%d",
+				       sa_p->ips_refhim);
+
+			if(sa_p->ips_out) {
+				len += ipsec_snprintf(buffer+len, length-len, " outif=%s:%d",
+						      sa_p->ips_out->name,
+						      sa_p->ips_transport_direct);
+			}
 #ifdef CONFIG_KLIPS_DEBUG
 			if(debug_xform) {
 			len += ipsec_snprintf(buffer+len, length-len, " reftable=%lu refentry=%lu",
@@ -391,7 +417,7 @@ ipsec_spi_get_info(char *buffer,
 
 			len += ipsec_snprintf(buffer+len, length-len, "\n");
 
-                        atomic_dec(&sa_p->ips_refcount);   
+                        ipsec_sa_put(sa_p);   
                        
                         if (len >= max_content) {
                                /* we've done all that can fit -- stop loops */
@@ -450,39 +476,35 @@ ipsec_spigrp_get_info(char *buffer,
 		     sa_p != NULL;
 		     sa_p = sa_p->ips_hnext)
 		{
-			atomic_inc(&sa_p->ips_refcount);
-			if(sa_p->ips_inext == NULL) {
-				sa_p2 = sa_p;
-				while(sa_p2 != NULL) {
-					atomic_inc(&sa_p2->ips_refcount);
-					sa_len = satot(&sa_p2->ips_said,
-						       'x', sa, sizeof(sa));
-					
-					len += ipsec_snprintf(buffer+len, length-len, "%s ",
-						       sa_len ? sa : " (error)");
-					atomic_dec(&sa_p2->ips_refcount);
-					sa_p2 = sa_p2->ips_onext;
-				}
-				len += ipsec_snprintf(buffer+len, length-len, "\n");
-                       }
-
-                       atomic_dec(&sa_p->ips_refcount);
-                                        
-                       if (len >= max_content) {
-                               /* we've done all that can fit -- stop loops */
-                               len = max_content;      /* truncate crap */
-                               goto done_spigrp_i;
-                       } else {
-                               const off_t pos = begin + len;
-
-                               if (pos <= offset) {
-                                       /* all is before first interesting character:
-                                        * discard, but note where we are.
-                                        */
+			sa_p2 = sa_p;
+			while(sa_p2 != NULL) {
+				struct ipsec_sa *sa2n;
+				sa_len = satot(&sa_p2->ips_said,
+					       'x', sa, sizeof(sa));
+				
+				len += ipsec_snprintf(buffer+len, length-len, "%s ",
+						      sa_len ? sa : " (error)");
+				
+				sa2n = sa_p2->ips_next;
+				sa_p2 = sa2n;
+			}
+			len += ipsec_snprintf(buffer+len, length-len, "\n");
+			
+			if (len >= max_content) {
+				/* we've done all that can fit -- stop loops */
+				len = max_content;      /* truncate crap */
+				goto done_spigrp_i;
+			} else {
+				const off_t pos = begin + len;
+				
+				if (pos <= offset) {
+					/* all is before first interesting character:
+					 * discard, but note where we are.
+					 */
                                         len = 0;
                                         begin = pos;
-                               }
-                       }
+				}
+			}
 		}
 	}
 
@@ -612,11 +634,7 @@ unsigned int natt_available = 1;
 #else
 unsigned int natt_available = 0;
 #endif
-#ifdef module_param
-module_param(natt_available, int, 0444);
-#else
-MODULE_PARM("natt_available","i");
-#endif
+module_param(natt_available,int,0644);
 
 IPSEC_PROCFS_DEBUG_NO_STATIC
 int
@@ -878,25 +896,25 @@ ipsec_proc_init()
 
         /* for 2.0 kernels */
 #if !defined(PROC_FS_2325) && !defined(PROC_FS_21)
-	error |= proc_register_dynamic(&proc_net, &ipsec_eroute);
-	error |= proc_register_dynamic(&proc_net, &ipsec_spi);
-	error |= proc_register_dynamic(&proc_net, &ipsec_spigrp);
-	error |= proc_register_dynamic(&proc_net, &ipsec_tncfg);
-	error |= proc_register_dynamic(&proc_net, &ipsec_version);
+	error |= proc_register_dynamic(&PROC_NET, &ipsec_eroute);
+	error |= proc_register_dynamic(&PROC_NET, &ipsec_spi);
+	error |= proc_register_dynamic(&PROC_NET, &ipsec_spigrp);
+	error |= proc_register_dynamic(&PROC_NET, &ipsec_tncfg);
+	error |= proc_register_dynamic(&PROC_NET, &ipsec_version);
 #ifdef CONFIG_KLIPS_DEBUG
-	error |= proc_register_dynamic(&proc_net, &ipsec_klipsdebug);
+	error |= proc_register_dynamic(&PROC_NET, &ipsec_klipsdebug);
 #endif /* CONFIG_KLIPS_DEBUG */
 #endif
 
 	/* for 2.2 kernels */
 #if !defined(PROC_FS_2325) && defined(PROC_FS_21)
-	error |= proc_register(proc_net, &ipsec_eroute);
-	error |= proc_register(proc_net, &ipsec_spi);
-	error |= proc_register(proc_net, &ipsec_spigrp);
-	error |= proc_register(proc_net, &ipsec_tncfg);
-	error |= proc_register(proc_net, &ipsec_version);
+	error |= proc_register(PROC_NET, &ipsec_eroute);
+	error |= proc_register(PROC_NET, &ipsec_spi);
+	error |= proc_register(PROC_NET, &ipsec_spigrp);
+	error |= proc_register(PROC_NET, &ipsec_tncfg);
+	error |= proc_register(PROC_NET, &ipsec_version);
 #ifdef CONFIG_KLIPS_DEBUG
-	error |= proc_register(proc_net, &ipsec_klipsdebug);
+	error |= proc_register(PROC_NET, &ipsec_klipsdebug);
 #endif /* CONFIG_KLIPS_DEBUG */
 #endif
 
@@ -908,7 +926,7 @@ ipsec_proc_init()
 	memset(&ipsec_ipv4_birth_packet, 0, sizeof(struct ipsec_birth_reply));
 	memset(&ipsec_ipv6_birth_packet, 0, sizeof(struct ipsec_birth_reply));
 
-	proc_net_ipsec_dir = proc_mkdir("ipsec", proc_net);
+	proc_net_ipsec_dir = proc_mkdir("ipsec", PROC_NET);
 	if(proc_net_ipsec_dir == NULL) {
 		/* no point in continuing */
 		return 1;
@@ -941,12 +959,12 @@ ipsec_proc_init()
 	}
 	
 	/* now create some symlinks to provide compatibility */
-	proc_symlink("ipsec_eroute", proc_net, "ipsec/eroute/all");
-	proc_symlink("ipsec_spi",    proc_net, "ipsec/spi/all");
-	proc_symlink("ipsec_spigrp", proc_net, "ipsec/spigrp/all");
-	proc_symlink("ipsec_tncfg",  proc_net, "ipsec/tncfg");
-	proc_symlink("ipsec_version",proc_net, "ipsec/version");
-	proc_symlink("ipsec_klipsdebug",proc_net,"ipsec/klipsdebug");
+	proc_symlink("ipsec_eroute", PROC_NET, "ipsec/eroute/all");
+	proc_symlink("ipsec_spi",    PROC_NET, "ipsec/spi/all");
+	proc_symlink("ipsec_spigrp", PROC_NET, "ipsec/spigrp/all");
+	proc_symlink("ipsec_tncfg",  PROC_NET, "ipsec/tncfg");
+	proc_symlink("ipsec_version",PROC_NET, "ipsec/version");
+	proc_symlink("ipsec_klipsdebug",PROC_NET,"ipsec/klipsdebug");
 
 #endif /* !PROC_FS_2325 */
 
@@ -1003,14 +1021,14 @@ ipsec_proc_cleanup()
 
 
 #ifdef CONFIG_KLIPS_DEBUG
-	remove_proc_entry("ipsec_klipsdebug", proc_net);
+	remove_proc_entry("ipsec_klipsdebug", PROC_NET);
 #endif /* CONFIG_KLIPS_DEBUG */
-	remove_proc_entry("ipsec_eroute",     proc_net);
-	remove_proc_entry("ipsec_spi",        proc_net);
-	remove_proc_entry("ipsec_spigrp",     proc_net);
-	remove_proc_entry("ipsec_tncfg",      proc_net);
-	remove_proc_entry("ipsec_version",    proc_net);
-	remove_proc_entry("ipsec",            proc_net);
+	remove_proc_entry("ipsec_eroute",     PROC_NET);
+	remove_proc_entry("ipsec_spi",        PROC_NET);
+	remove_proc_entry("ipsec_spigrp",     PROC_NET);
+	remove_proc_entry("ipsec_tncfg",      PROC_NET);
+	remove_proc_entry("ipsec_version",    PROC_NET);
+	remove_proc_entry("ipsec",            PROC_NET);
 #endif /* 2.4 kernel */
 }
 
