@@ -47,7 +47,6 @@
 
 #ifdef NET_21
 # include <linux/in6.h>
-# define ip_chk_addr inet_addr_type
 # define IS_MYADDR RTN_LOCAL
 # include <net/dst.h>
 # undef dev_kfree_skb
@@ -57,6 +56,7 @@
 
 #include <net/icmp.h>		/* icmp_send() */
 #include <net/ip.h>
+#include <net/arp.h>
 #ifdef NETDEV_23
 # include <linux/netfilter_ipv4.h>
 #endif /* NETDEV_23 */
@@ -89,10 +89,6 @@
 
 static __u32 zeroes[64];
 
-#ifdef CONFIG_KLIPS_DEBUG
-int debug_tunnel = 0;
-#endif /* CONFIG_KLIPS_DEBUG */
-
 DEBUG_NO_STATIC int
 ipsec_tunnel_open(struct net_device *dev)
 {
@@ -204,7 +200,7 @@ ipsec_tunnel_SAlookup(struct ipsec_xmit_state *ixs)
 	 * The spinlock is to prevent any other process from accessing or deleting
 	 * the eroute while we are using and updating it.
 	 */
-	spin_lock(&eroute_lock);
+	spin_lock_bh(&eroute_lock);
 	
 	ixs->eroute = ipsec_findroute(&ixs->matcher);
 
@@ -415,7 +411,7 @@ ipsec_tunnel_SAlookup(struct ipsec_xmit_state *ixs)
 			ixs->eroute->er_last = ixs->skb;
 			ixs->skb = NULL;
 			ixs->stats->tx_dropped++;
-			spin_unlock(&eroute_lock);
+			spin_unlock_bh(&eroute_lock);
 			return IPSEC_XMIT_STOLEN;
 		}
 		ixs->outgoing_said = ixs->eroute->er_said;
@@ -442,7 +438,7 @@ ipsec_tunnel_SAlookup(struct ipsec_xmit_state *ixs)
 					       "Failed, tried to allocate %d bytes for source ident.\n", 
 					       len);
 					ixs->stats->tx_dropped++;
-					spin_unlock(&eroute_lock);
+					spin_unlock_bh(&eroute_lock);
 					return IPSEC_XMIT_ERRMEMALLOC;
 				}
 				memcpy(ixs->ips.ips_ident_s.data, ixs->eroute->er_ident_s.data, len);
@@ -462,7 +458,7 @@ ipsec_tunnel_SAlookup(struct ipsec_xmit_state *ixs)
 					       "Failed, tried to allocate %d bytes for dest ident.\n", 
 					       len);
 					ixs->stats->tx_dropped++;
-					spin_unlock(&eroute_lock);
+					spin_unlock_bh(&eroute_lock);
 					return IPSEC_XMIT_ERRMEMALLOC;
 				}
 				memcpy(ixs->ips.ips_ident_d.data, ixs->eroute->er_ident_d.data, len);
@@ -470,7 +466,7 @@ ipsec_tunnel_SAlookup(struct ipsec_xmit_state *ixs)
 		}
 	}
 
-	spin_unlock(&eroute_lock);
+	spin_unlock_bh(&eroute_lock);
 	return IPSEC_XMIT_OK;
 }
 
@@ -503,52 +499,7 @@ ipsec_tunnel_restore_hard_header(struct ipsec_xmit_state*ixs)
 			}
 		}
 	}
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-	if (ixs->natt_type && ixs->natt_head) {
-		struct iphdr *ipp = ip_hdr(ixs->skb);
-		struct udphdr *udp;
-		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-			    "klips_debug:ipsec_tunnel_start_xmit: "
-			    "encapsuling packet into UDP (NAT-Traversal) (%d %d)\n",
-			    ixs->natt_type, ixs->natt_head);
 
-		ixs->iphlen = ipp->ihl << 2;
-		ipp->tot_len =
-			htons(ntohs(ipp->tot_len) + ixs->natt_head);
-		if(skb_tailroom(ixs->skb) < ixs->natt_head) {
-			printk(KERN_WARNING "klips_error:ipsec_tunnel_start_xmit: "
-				"tried to skb_put %d, %d available. "
-				"This should never happen, please report.\n",
-				ixs->natt_head,
-				skb_tailroom(ixs->skb));
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_ESPUDP;
-		}
-		skb_put(ixs->skb, ixs->natt_head);
-
-		udp = (struct udphdr *)((char *)ipp + ixs->iphlen);
-
-		/* move ESP hdr after UDP hdr */
-		memmove((void *)((char *)udp + ixs->natt_head),
-			(void *)(udp),
-			ntohs(ipp->tot_len) - ixs->iphlen - ixs->natt_head);
-
-		/* clear UDP & Non-IKE Markers (if any) */
-		memset(udp, 0, ixs->natt_head);
-
-		/* fill UDP with usefull informations ;-) */
-		udp->source = htons(ixs->natt_sport);
-		udp->dest = htons(ixs->natt_dport);
-		udp->len = htons(ntohs(ipp->tot_len) - ixs->iphlen);
-
-		/* set protocol */
-		ipp->protocol = IPPROTO_UDP;
-
-		/* fix IP checksum */
-		ipp->check = 0;
-		ipp->check = ip_fast_csum((unsigned char *)ipp, ipp->ihl);
-	}
-#endif	
 	KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
 		    "klips_debug:ipsec_xmit_restore_hard_header: "
 		    "With hard_header, final head,tailroom: %d,%d\n",
@@ -558,130 +509,6 @@ ipsec_tunnel_restore_hard_header(struct ipsec_xmit_state*ixs)
 	return IPSEC_XMIT_OK;
 }
 
-enum ipsec_xmit_value
-ipsec_tunnel_send(struct ipsec_xmit_state*ixs)
-{
-	int err;
-#ifdef NETDEV_25
-	struct flowi fl;
-#endif
-  
-	/* new route/dst cache code from James Morris */
-	ixs->skb->dev = ixs->physdev;
-#ifdef NETDEV_25
- 	fl.oif = ixs->physdev->iflink;
- 	fl.nl_u.ip4_u.daddr = ip_hdr(ixs->skb)->daddr;
-	fl.nl_u.ip4_u.saddr = ixs->pass ? 0 : ip_hdr(ixs->skb)->saddr;
-	fl.nl_u.ip4_u.tos = RT_TOS(ip_hdr(ixs->skb)->tos);
-	fl.proto = ip_hdr(ixs->skb)->protocol;
- 	if ((ixs->error = ip_route_output_key(&ixs->route, &fl))) {
-#else
-	/*skb_orphan(ixs->skb);*/
-	if((ixs->error = ip_route_output(&ixs->route,
-				    ixs->skb->nh.iph->daddr,
-				    ixs->pass ? 0 : ip_hdr(ixs->skb)->saddr,
-				    RT_TOS(ip_hdr(ixs->skb)->tos),
-                                    /* mcr->rgb: should this be 0 instead? */
-				    ixs->physdev->iflink))) {
-#endif
-		ixs->stats->tx_errors++;
-		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-			    "klips_debug:ipsec_xmit_send: "
-			    "ip_route_output failed with error code %d, rt->u.dst.dev=%s, dropped\n",
-			    ixs->error,
-			    ixs->route->u.dst.dev->name);
-		return IPSEC_XMIT_ROUTEERR;
-	}
-
-	if(ixs->dev == ixs->route->u.dst.dev) {
-		ip_rt_put(ixs->route);
-		/* This is recursion, drop it. */
-		ixs->stats->tx_errors++;
-		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-			    "klips_debug:ipsec_xmit_send: "
-			    "suspect recursion, dev=rt->u.dst.dev=%s, dropped\n",
-			    ixs->dev->name);
-		return IPSEC_XMIT_RECURSDETECT;
-	}
-	dst_release(ixs->skb->dst);
-	ixs->skb->dst = &ixs->route->u.dst;
-
-	ixs->stats->tx_bytes += ixs->skb->len;
-	if(ixs->skb->len < skb_network_header(ixs->skb) - ixs->skb->data) {
-		ixs->stats->tx_errors++;
-		printk(KERN_WARNING
-		       "klips_error:ipsec_xmit_send: "
-		       "tried to __skb_pull nh-data=%ld, %d available.  This should never happen, please report.\n",
-		       (unsigned long)(skb_network_header(ixs->skb) - ixs->skb->data),
-		       ixs->skb->len);
-		return IPSEC_XMIT_PUSHPULLERR;
-	}
-	__skb_pull(ixs->skb, skb_network_header(ixs->skb) - ixs->skb->data);
-#ifdef SKB_RESET_NFCT
-	if(!ixs->pass) {
-		nf_conntrack_put(ixs->skb->nfct);
-		ixs->skb->nfct = NULL;
-	}
-#if defined(CONFIG_NETFILTER_DEBUG) && defined(HAVE_SKB_NF_DEBUG)
-	ixs->skb->nf_debug = 0;
-#endif /* CONFIG_NETFILTER_DEBUG */
-#endif /* SKB_RESET_NFCT */
-	KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-		    "klips_debug:ipsec_xmit_send: "
-		    "...done, calling ip_send() on device:%s\n",
-		    ixs->skb->dev ? ixs->skb->dev->name : "NULL");
-	KLIPS_IP_PRINT(debug_tunnel & DB_TN_XMIT, ip_hdr(ixs->skb));
-
-	if(ixs->pass) {
-		err = ipsec_tunnel_xmit2(ixs->skb);
-	} else {
-		err = NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, ixs->skb, NULL, ixs->route->u.dst.dev,
-			      ipsec_tunnel_xmit2);
-	}
-	if(err != NET_XMIT_SUCCESS && err != NET_XMIT_CN) {
-		if(net_ratelimit())
-			printk(KERN_ERR
-			       "klips_error:ipsec_xmit_send: "
-			       "ip_send() failed, err=%d\n", 
-			       -err);
-		ixs->stats->tx_errors++;
-		ixs->stats->tx_aborted_errors++;
-		ixs->skb = NULL;
-		return IPSEC_XMIT_IPSENDFAILURE;
-	}
-
-	ixs->stats->tx_packets++;
-
-	ixs->skb = NULL;
-	
-	return IPSEC_XMIT_OK;
-}
-
-void
-ipsec_tunnel_cleanup(struct ipsec_xmit_state*ixs)
-{
-#if defined(HAS_NETIF_QUEUE) || defined (HAVE_NETIF_QUEUE)
-	netif_wake_queue(ixs->dev);
-#else /* defined(HAS_NETIF_QUEUE) || defined (HAVE_NETIF_QUEUE) */
-	ixs->dev->tbusy = 0;
-#endif /* defined(HAS_NETIF_QUEUE) || defined (HAVE_NETIF_QUEUE) */
-	if(ixs->saved_header) {
-		kfree(ixs->saved_header);
-	}
-	if(ixs->skb) {
-		dev_kfree_skb(ixs->skb, FREE_WRITE);
-	}
-	if(ixs->oskb) {
-		dev_kfree_skb(ixs->oskb, FREE_WRITE);
-	}
-	if (ixs->ips.ips_ident_s.data) {
-		kfree(ixs->ips.ips_ident_s.data);
-	}
-	if (ixs->ips.ips_ident_d.data) {
-		kfree(ixs->ips.ips_ident_d.data);
-	}
-}
-
 /* management of buffers */
 static struct ipsec_xmit_state * ipsec_xmit_state_new (void);
 static void ipsec_xmit_state_delete (struct ipsec_xmit_state *ixs);
@@ -697,6 +524,13 @@ ipsec_tunnel_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	struct ipsec_xmit_state *ixs = NULL;
 	enum ipsec_xmit_value stat;
 
+	KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+		    "\n\nipsec_tunnel_start_xmit: STARTING");
+
+#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
+	ixs->natt_type = 0, ixs->natt_head = 0;
+	ixs->natt_sport = 0, ixs->natt_dport = 0;
+#endif
         stat = IPSEC_XMIT_ERRMEMALLOC;
         ixs = ipsec_xmit_state_new ();
         if (! ixs) {
@@ -749,7 +583,7 @@ ipsec_tunnel_start_xmit(struct sk_buff *skb, struct net_device *dev)
 		ixs->matcher.sen_proto = ixs->iph->protocol;
 		ipsec_extract_ports(ixs->iph, &ixs->matcher);
 
-		spin_lock(&eroute_lock);
+		spin_lock_bh(&eroute_lock);
 		ixs->eroute = ipsec_findroute(&ixs->matcher);
 		if(ixs->eroute) {
 			ixs->outgoing_said = ixs->eroute->er_said;
@@ -757,7 +591,7 @@ ipsec_tunnel_start_xmit(struct sk_buff *skb, struct net_device *dev)
 			ixs->eroute->er_count++;
 			ixs->eroute->er_lasttime = jiffies/HZ;
 		}
-		spin_unlock(&eroute_lock);
+		spin_unlock_bh(&eroute_lock);
 
 		KLIPS_PRINT((debug_tunnel & DB_TN_XMIT) &&
 			    /* ((ixs->orgdst != ixs->newdst) || (ixs->orgsrc != ixs->newsrc)) */
@@ -772,6 +606,11 @@ ipsec_tunnel_start_xmit(struct sk_buff *skb, struct net_device *dev)
 		ixs->outgoing_said.dst.u.v4.sin_addr.s_addr &&
 		ixs->eroute);
 	
+	stat = ipsec_nat_encap(ixs);
+	if(stat != IPSEC_XMIT_OK) {
+		goto cleanup;
+	}
+
 	stat = ipsec_tunnel_restore_hard_header(ixs);
 	if(stat != IPSEC_XMIT_OK) {
 		goto cleanup;
@@ -781,7 +620,7 @@ ipsec_tunnel_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	stat = ipsec_tunnel_send(ixs);
 
  cleanup:
-	ipsec_tunnel_cleanup(ixs);
+	ipsec_xmit_cleanup(ixs);
 
         ipsec_xmit_state_delete (ixs);
 alloc_error:
@@ -860,7 +699,12 @@ ipsec_tunnel_hard_header(struct sk_buff *skb, struct net_device *dev,
 	if(type != ETH_P_IPV6) {
 		/* execute this only, if we don't have to build the
 		   header for a IPv6 packet */
-		if(!prv->hard_header) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+               if(!prv->header_ops->create)
+#else
+               if(!prv->hard_header)
+#endif
+               {
 			KLIPS_PRINT(debug_tunnel & DB_TN_REVEC,
 				    "klips_debug:ipsec_tunnel_hard_header: "
 				    "physical device has been detached, packet dropped 0p%p->0p%p len=%d type=%d dev=%s->NULL ",
@@ -913,7 +757,11 @@ ipsec_tunnel_hard_header(struct sk_buff *skb, struct net_device *dev,
 	}                                                                       
 	tmp = skb->dev;
 	skb->dev = prv->dev;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+       ret = prv->header_ops->create(skb, prv->dev, type, (void *)daddr, (void *)saddr, len);
+#else
 	ret = prv->hard_header(skb, prv->dev, type, (void *)daddr, (void *)saddr, len);
+#endif
 	skb->dev = tmp;
 	return ret;
 }
@@ -956,8 +804,12 @@ ipsec_tunnel_rebuild_header(void *buff, struct net_device *dev,
 		stats->tx_dropped++;
 		return -ENODEV;
 	}
-
-	if(!prv->rebuild_header) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	if(!prv->header_ops->rebuild)
+#else
+	if(!prv->rebuild_header)
+#endif
+	{
 		KLIPS_PRINT(debug_tunnel & DB_TN_REVEC,
 			    "klips_debug:ipsec_tunnel_rebuild_header: "
 			    "physical device has been detached, packet dropped skb->dev=%s->NULL ",
@@ -994,12 +846,16 @@ ipsec_tunnel_rebuild_header(void *buff, struct net_device *dev,
 #endif /* NET_21 */
 	tmp = skb->dev;
 	skb->dev = prv->dev;
-	
-#ifdef NET_21
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	ret = prv->header_ops->rebuild(skb);
+#else
+# ifdef NET_21
 	ret = prv->rebuild_header(skb);
-#else /* NET_21 */
+# else /* NET_21 */
 	ret = prv->rebuild_header(buff, prv->dev, raddr, skb);
-#endif /* NET_21 */
+# endif /* NET_21 */
+#endif
 	skb->dev = tmp;
 	return ret;
 }
@@ -1139,7 +995,12 @@ ipsec_tunnel_cache_update(struct hh_cache *hh, struct net_device *dev, unsigned
 		return;
 	}
 
-	if(!prv->header_cache_update) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	if(!prv->header_ops->cache_update)
+#else
+	if(!prv->header_cache_update)
+#endif
+       {
 		KLIPS_PRINT(debug_tunnel & DB_TN_REVEC,
 			    "klips_debug:ipsec_tunnel_cache_update: "
 			    "physical device has been detached, cannot set - skb->dev=%s->NULL\n",
@@ -1150,10 +1011,22 @@ ipsec_tunnel_cache_update(struct hh_cache *hh, struct net_device *dev, unsigned
 	KLIPS_PRINT(debug_tunnel & DB_TN_REVEC,
 		    "klips_debug:ipsec_tunnel: "
 		    "Revectored cache_update\n");
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	prv->header_ops->cache_update(hh, prv->dev, haddr);
+#else
 	prv->header_cache_update(hh, prv->dev, haddr);
+#endif
 	return;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+const struct header_ops ipsec_tunnel_header_ops = {
+	.create         = ipsec_tunnel_hard_header,
+	.rebuild        = ipsec_tunnel_rebuild_header,
+	.cache_update   = ipsec_tunnel_cache_update,
+};
+#endif
+
 #ifdef NET_21
 DEBUG_NO_STATIC int
 ipsec_tunnel_neigh_setup(struct neighbour *n)
@@ -1214,10 +1087,16 @@ ipsec_tunnel_attach(struct net_device *dev, struct net_device *physdev)
 	prv->hard_start_xmit = physdev->hard_start_xmit;
 	prv->get_stats = physdev->get_stats;
 
-#ifdef HAVE_DEV_HARD_HEADER
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	if (physdev->header_ops) {
+		prv->header_ops = physdev->header_ops;
+		dev->header_ops = &ipsec_tunnel_header_ops;
+	} else
+		dev->header_ops = NULL;
+#else
 	if (physdev->hard_header) {
 		prv->hard_header = physdev->hard_header;
-		dev->hard_header = ipsec_tunnel_hard_header;
+		dev->hard_header = &ipsec_tunnel_hard_header;
 	} else
 		dev->hard_header = NULL;
 	
@@ -1227,12 +1106,6 @@ ipsec_tunnel_attach(struct net_device *dev, struct net_device *physdev)
 	} else
 		dev->rebuild_header = NULL;
 	
-	if (physdev->set_mac_address) {
-		prv->set_mac_address = physdev->set_mac_address;
-		dev->set_mac_address = ipsec_tunnel_set_mac_address;
-	} else
-		dev->set_mac_address = NULL;
-	
 #ifndef NET_21
 	if (physdev->header_cache_bind) {
 		prv->header_cache_bind = physdev->header_cache_bind;
@@ -1246,7 +1119,13 @@ ipsec_tunnel_attach(struct net_device *dev, struct net_device *physdev)
 		dev->header_cache_update = ipsec_tunnel_cache_update;
 	} else
 		dev->header_cache_update = NULL;
-#endif /* HAVE_DEV_HARD_HEADER */
+#endif
+
+	if (physdev->set_mac_address) {
+		prv->set_mac_address = physdev->set_mac_address;
+		dev->set_mac_address = ipsec_tunnel_set_mac_address;
+	} else
+		dev->set_mac_address = NULL;
 
 	dev->hard_header_len = physdev->hard_header_len;
 
@@ -1316,43 +1195,38 @@ ipsec_tunnel_detach(struct net_device *dev)
 	prv->hard_start_xmit = NULL;
 	prv->get_stats = NULL;
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	prv->header_ops = NULL;
+#else
 	prv->hard_header = NULL;
-#ifdef DETACH_AND_DOWN
-	dev->hard_header = NULL;
-#endif /* DETACH_AND_DOWN */
-	
 	prv->rebuild_header = NULL;
-#ifdef DETACH_AND_DOWN
-	dev->rebuild_header = NULL;
-#endif /* DETACH_AND_DOWN */
-	
-	prv->set_mac_address = NULL;
-#ifdef DETACH_AND_DOWN
-	dev->set_mac_address = NULL;
-#endif /* DETACH_AND_DOWN */
-	
+	prv->header_cache_update = NULL;
 #ifndef NET_21
 	prv->header_cache_bind = NULL;
-#ifdef DETACH_AND_DOWN
-	dev->header_cache_bind = NULL;
-#endif /* DETACH_AND_DOWN */
-#endif /* !NET_21 */
-
-	prv->header_cache_update = NULL;
-#ifdef DETACH_AND_DOWN
-	dev->header_cache_update = NULL;
-#endif /* DETACH_AND_DOWN */
-
-#ifdef NET_21
-/*	prv->neigh_setup        = NULL; */
-#ifdef DETACH_AND_DOWN
-	dev->neigh_setup        = NULL;
-#endif /* DETACH_AND_DOWN */
-#endif /* NET_21 */
+#else
+/*     prv->neigh_setup        = NULL; */
+#endif
+#endif
+	prv->set_mac_address = NULL;
 	dev->hard_header_len = 0;
+
 #ifdef DETACH_AND_DOWN
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	dev->header_ops = NULL;
+#else
+	dev->hard_header = NULL;
+	dev->rebuild_header = NULL;
+	dev->header_cache_update = NULL;
+#ifndef NET_21
+	dev->header_cache_bind = NULL;
+#else
+	dev->neigh_setup        = NULL;
+#endif
+#endif
+	dev->set_mac_address = NULL;
 	dev->mtu = 0;
 #endif /* DETACH_AND_DOWN */
+
 	prv->mtu = 0;
 	for (i=0; i<MAX_ADDR_LEN; i++) {
 		dev->dev_addr[i] = 0;
@@ -1720,23 +1594,23 @@ ipsec_tunnel_init(struct net_device *dev)
 
 	dev->set_multicast_list = NULL;
 	dev->do_ioctl		= ipsec_tunnel_ioctl;
-#ifdef HAVE_DEV_HARD_HEADER
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	dev->header_ops         = NULL;
+#else
 	dev->hard_header	= NULL;
 	dev->rebuild_header 	= NULL;
-	dev->header_cache_update= NULL;
-#endif
 	dev->set_mac_address 	= NULL;
 #ifndef NET_21
 	dev->header_cache_bind 	= NULL;
 #endif /* !NET_21 */
+	dev->header_cache_update= NULL;
+#endif
 
 #ifdef NET_21
 /*	prv->neigh_setup        = NULL; */
 	dev->neigh_setup        = ipsec_tunnel_neigh_setup_dev;
 #endif /* NET_21 */
-#ifdef HAVE_DEV_HARD_HEADER
 	dev->hard_header_len 	= 0;
-#endif
 	dev->mtu		= 0;
 	dev->addr_len		= 0;
 	dev->type		= ARPHRD_VOID; /* ARPHRD_TUNNEL; */ /* ARPHRD_ETHER; */
@@ -1746,18 +1620,6 @@ ipsec_tunnel_init(struct net_device *dev)
 	/* New-style flags. */
 	dev->flags		= IFF_NOARP /* 0 */ /* Petr Novak */;
 
-#if 0
-#ifdef NET_21
-	dev_init_buffers(dev);
-#else /* NET_21 */
-	dev->family		= AF_INET;
-	dev->pa_addr		= 0;
-	dev->pa_brdaddr 	= 0;
-	dev->pa_mask		= 0;
-	dev->pa_alen		= 4;
-#endif /* NET_21 */
-#endif
-
 	/* We're done.  Have I forgotten anything? */
 	return 0;
 }
@@ -1773,14 +1635,105 @@ ipsec_tunnel_probe(struct net_device *dev)
 	return 0;
 }
 
-struct net_device *ipsecdevices[IPSEC_NUM_IF];
+#ifdef alloc_netdev
+static void ipsec_tunnel_netdev_setup(struct net_device *dev)
+{
+}
+#endif
+
+struct net_device *ipsecdevices[IPSEC_NUM_IFMAX];
+int ipsecdevices_max=-1;
+
+
+int
+ipsec_tunnel_createnum(int ifnum)
+{
+	char name[IFNAMSIZ];
+	struct net_device *dev_ipsec;
+	int vifentry;
+
+	if(ifnum > IPSEC_NUM_IFMAX) {
+		return -ENOENT;
+	}
+
+	if(ipsecdevices[ifnum]!=NULL) {
+		return -EEXIST;
+	}
+	
+	/* no identical device */
+	if(ifnum > ipsecdevices_max) {
+		ipsecdevices_max=ifnum;
+	}
+	vifentry = ifnum;
+	
+	KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
+		    "klips_debug:ipsec_tunnel_init_devices: "
+		    "creating and registering IPSEC_NUM_IF=%u device\n",
+		    ifnum);
+
+	sprintf(name, IPSEC_DEV_FORMAT, ifnum);
+#ifdef alloc_netdev
+	dev_ipsec = alloc_netdev(0, name, ipsec_tunnel_netdev_setup);
+#else
+	dev_ipsec = (struct net_device*)kmalloc(sizeof(struct net_device), GFP_KERNEL);
+#endif
+	if (dev_ipsec == NULL) {
+		printk(KERN_ERR "klips_debug:ipsec_tunnel_init_devices: "
+		       "failed to allocate memory for device %s, quitting device init.\n",
+		       name);
+		return -ENOMEM;
+	}
+#ifndef alloc_netdev
+	memset((caddr_t)dev_ipsec, 0, sizeof(struct net_device));
+#ifdef NETDEV_23
+	strncpy(dev_ipsec->name, name, sizeof(dev_ipsec->name));
+#else /* NETDEV_23 */
+	dev_ipsec->name = (char*)kmalloc(IFNAMSIZ, GFP_KERNEL);
+	if (dev_ipsec->name == NULL) {
+		KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
+			    "klips_debug:ipsec_tunnel_init_devices: "
+			    "failed to allocate memory for device %s name, quitting device init.\n",
+			    name);
+		return -ENOMEM;
+	}
+	memset((caddr_t)dev_ipsec->name, 0, IFNAMSIZ);
+	strncpy(dev_ipsec->name, name, IFNAMSIZ);
+#endif /* NETDEV_23 */
+#ifdef PAUL_FIXME
+	dev_ipsec->next = NULL;
+#endif
+#endif /* alloc_netdev */
+	dev_ipsec->init = &ipsec_tunnel_probe;
+	KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
+		    "klips_debug:ipsec_tunnel_init_devices: "
+		    "registering device %s\n",
+		    dev_ipsec->name);
+	
+	/* reference and hold the device reference */
+	dev_hold(dev_ipsec);
+	ipsecdevices[vifentry]=dev_ipsec;
+	
+	if (register_netdev(dev_ipsec) != 0) {
+		KLIPS_PRINT(1 || debug_tunnel & DB_TN_INIT,
+			    "klips_debug:ipsec_tunnel_init_devices: "
+			    "registering device %s failed, quitting device init.\n",
+			    dev_ipsec->name);
+		return -EIO;
+	} else {
+		KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
+			    "klips_debug:ipsec_tunnel_init_devices: "
+			    "registering device %s succeeded, continuing...\n",
+			    dev_ipsec->name);
+	}
+	return 0;
+}
+	
 
 int 
 ipsec_tunnel_init_devices(void)
 {
 	int i;
-	char name[IFNAMSIZ];
-	struct net_device *dev_ipsec;
+	int error;
 	
 	KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
 		    "klips_debug:ipsec_tunnel_init_devices: "
@@ -1790,59 +1743,67 @@ ipsec_tunnel_init_devices(void)
 		    IFNAMSIZ);
 
 	for(i = 0; i < IPSEC_NUM_IF; i++) {
-		sprintf(name, IPSEC_DEV_FORMAT, i);
-		dev_ipsec = (struct net_device*)kmalloc(sizeof(struct net_device), GFP_KERNEL);
-		if (dev_ipsec == NULL) {
-			KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
-				    "klips_debug:ipsec_tunnel_init_devices: "
-				    "failed to allocate memory for device %s, quitting device init.\n",
-				    name);
-			return -ENOMEM;
-		}
-		memset((caddr_t)dev_ipsec, 0, sizeof(struct net_device));
-#ifdef NETDEV_23
-		strncpy(dev_ipsec->name, name, sizeof(dev_ipsec->name));
-#else /* NETDEV_23 */
-		dev_ipsec->name = (char*)kmalloc(IFNAMSIZ, GFP_KERNEL);
-		if (dev_ipsec->name == NULL) {
-			KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
-				    "klips_debug:ipsec_tunnel_init_devices: "
-				    "failed to allocate memory for device %s name, quitting device init.\n",
-				    name);
-			return -ENOMEM;
-		}
-		memset((caddr_t)dev_ipsec->name, 0, IFNAMSIZ);
-		strncpy(dev_ipsec->name, name, IFNAMSIZ);
-#endif /* NETDEV_23 */
-#ifdef HAVE_DEV_NEXT
-		dev_ipsec->next = NULL;
-#endif
-		dev_ipsec->init = &ipsec_tunnel_probe;
-		KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
-			    "klips_debug:ipsec_tunnel_init_devices: "
-			    "registering device %s\n",
-			    dev_ipsec->name);
-
-		/* reference and hold the device reference */
-		dev_hold(dev_ipsec);
-		ipsecdevices[i]=dev_ipsec;
-
-		if (register_netdev(dev_ipsec) != 0) {
-			KLIPS_PRINT(1 || debug_tunnel & DB_TN_INIT,
-				    "klips_debug:ipsec_tunnel_init_devices: "
-				    "registering device %s failed, quitting device init.\n",
-				    dev_ipsec->name);
-			return -EIO;
-		} else {
-			KLIPS_PRINT(debug_tunnel & DB_TN_INIT,
-				    "klips_debug:ipsec_tunnel_init_devices: "
-				    "registering device %s succeeded, continuing...\n",
-				    dev_ipsec->name);
-		}
+		error = ipsec_tunnel_createnum(i);
+		
+		if(error) break;
 	}
 	return 0;
 }
 
+int
+ipsec_tunnel_deletenum(int vifnum)
+{
+	struct net_device *dev_ipsec;
+	
+	if(vifnum > IPSEC_NUM_IFMAX) {
+		return -ENOENT;
+	}
+
+	dev_ipsec = ipsecdevices[vifnum];
+	if(dev_ipsec == NULL) {
+		return -ENOENT;
+	}
+
+	/* release reference */
+	ipsecdevices[vifnum]=NULL;
+	ipsec_dev_put(dev_ipsec);
+	
+	KLIPS_PRINT(debug_tunnel, "Unregistering %s (refcnt=%d)\n",
+		    dev_ipsec->name,
+		    atomic_read(&dev_ipsec->refcnt));
+	unregister_netdev(dev_ipsec);
+	KLIPS_PRINT(debug_tunnel, "Unregisted %s\n", dev_ipsec->name);
+#ifdef alloc_netdev
+	free_netdev(dev_ipsec);
+#else
+#ifndef NETDEV_23
+	kfree(dev_ipsec->name);
+	dev_ipsec->name=NULL;
+#endif /* !NETDEV_23 */
+	kfree(dev_ipsec->priv);
+#endif /* alloc_netdev */
+	dev_ipsec->priv=NULL;
+
+	return 0;
+}
+
+
+struct net_device *
+ipsec_tunnel_get_device(int vifnum)
+{
+	struct net_device *nd;
+	
+	if(vifnum < ipsecdevices_max) {
+		nd = ipsecdevices[vifnum];
+
+		if(nd) dev_hold(nd);
+		return nd;
+	} else {
+		return NULL;
+	}
+}
+
+
 /* void */
 int
 ipsec_tunnel_cleanup_devices(void)
@@ -1901,7 +1862,7 @@ ipsec_xmit_state_cache_init (void)
 	/* ixs_cache_allocator = KMEM_CACHE(ipsec_ixs,0); */
         ixs_cache_allocator = kmem_cache_create ("ipsec_ixs",
                 sizeof (struct ipsec_xmit_state), 0,
-                0, NULL );
+                0, NULL);
 #else
         ixs_cache_allocator = kmem_cache_create ("ipsec_ixs",
                 sizeof (struct ipsec_xmit_state), 0,
