@@ -56,6 +56,11 @@ char klipsdebug_c_version[] = "RCSID $Id: klipsdebug.c,v 1.58 2005/08/18 14:04:3
 #endif /* CONFIG_KLIPS_DEBUG */
 #include "openswan/ipsec_tunnel.h"
 
+#include <stdio.h>
+#include <getopt.h>
+
+#include "oswlog.h"
+
 __u32 bigbuf[1024];
 char *program_name;
 
@@ -118,7 +123,7 @@ main(int argc, char **argv)
 	int em_db_rj, em_db_es, em_db_ah, em_db_rx, em_db_ky;
 	int em_db_gz, em_db_vb;
 
-	struct sadb_ext *extensions[SADB_EXT_MAX + 1];
+	struct sadb_ext *extensions[K_SADB_EXT_MAX + 1];
 	struct sadb_msg *pfkey_msg;
 	
 	em_db_tn=em_db_nl=em_db_xf=em_db_er=em_db_sp=0;
