@@ -425,7 +425,7 @@ format_end(char *buf
     char host_port[sizeof(":65535")];
     char host_id[IDTOA_BUF + 2];
     char hop[ADDRTOT_BUF];
-    char endopts[sizeof("MS+MC+XS+XC")+1];
+    char endopts[sizeof("MS+MC+XS+XC+Sxx")+1];
     const char *hop_sep = "";
     const char *open_brackets  = "";
     const char *close_brackets = "";
@@ -542,7 +542,7 @@ format_end(char *buf
        || this->xauth_server || this->xauth_client
        || this->sendcert != cert_defaultcertpolicy)
     {
-	const char *plus = "";
+	const char *plus = "+";
 	endopts[0]='\0';
 
 	if(id_obrackets && id_obrackets[0]=='[')
@@ -554,34 +554,29 @@ format_end(char *buf
 	}
 
 	if(this->modecfg_server) {
-	    strncat(endopts, plus, sizeof(endopts));
-	    strncat(endopts, "MS", sizeof(endopts));
-	    plus="+";
+	    strncat(endopts, "MS", sizeof(endopts) - strlen(endopts)-1);
 	}
 
 	if(this->modecfg_client) {
-	    strncat(endopts, plus, sizeof(endopts));
-	    strncat(endopts, "MC", sizeof(endopts));
-	    plus="+";
+	    strncat(endopts, plus, sizeof(endopts) - strlen(endopts)-1);
+	    strncat(endopts, "MC", sizeof(endopts) - strlen(endopts)-1);
 	}
 
 	if(this->xauth_server) {
-	    strncat(endopts, plus, sizeof(endopts));
-	    strncat(endopts, "XS", sizeof(endopts));
-	    plus="+";
+	    strncat(endopts, plus, sizeof(endopts) - strlen(endopts)-1);
+	    strncat(endopts, "XS", sizeof(endopts) - strlen(endopts)-1);
 	}
 	    
 	if(this->xauth_client) {
-	    strncat(endopts, plus, sizeof(endopts));
-	    strncat(endopts, "XC", sizeof(endopts));
-	    plus="+";
+	    strncat(endopts, plus, sizeof(endopts) - strlen(endopts)-1);
+	    strncat(endopts, "XC", sizeof(endopts) - strlen(endopts)-1);
 	}
 
 	{
 	    const char *send_cert = "";
 	    char s[32];
 
-	    send_cert="";
+	    send_cert=""; // Length 3 because cert.type is 1-11
 	    
 	    switch(this->sendcert) {
 	    case cert_neversend:
@@ -598,9 +593,8 @@ format_end(char *buf
 		send_cert=s;
 		break;
 	    }
-	    strncat(endopts, plus, sizeof(endopts));
-	    strncat(endopts, send_cert, sizeof(endopts));
-	    plus="+";
+	    strncat(endopts, plus, sizeof(endopts) - strlen(endopts)-1);
+	    strncat(endopts, send_cert, sizeof(endopts) - strlen(endopts)-1);
 	}
     }
 #endif
@@ -892,6 +886,15 @@ extract_end(struct end *dst, const struct whack_end *src, const char *which)
     dst->host_nexthop = src->host_nexthop;
     dst->host_srcip = src->host_srcip;
     dst->client = src->client;
+
+#ifdef HAVE_SIN_LEN
+    /* XXX need to fix this for v6 */
+    dst->client.addr.u.v4.sin_len  = sizeof(struct sockaddr_in);
+    dst->host_addr.u.v4.sin_len    = sizeof(struct sockaddr_in);
+    dst->host_nexthop.u.v4.sin_len = sizeof(struct sockaddr_in);
+    dst->host_srcip.u.v4.sin_len   = sizeof(struct sockaddr_in);
+#endif
+    
 #ifdef MODECFG
     dst->modecfg_server = src->modecfg_server;
     dst->modecfg_client = src->modecfg_client;
@@ -1143,14 +1146,24 @@ add_connection(const struct whack_message *wm)
 	{
 		DBG(DBG_CONTROL, DBG_log("from whack: got --esp=%s", wm->esp ? wm->esp: "NULL"));
 
-		if(c->policy & POLICY_ENCRYPT) {
-		    c->alg_info_esp = alg_info_esp_create_from_str(wm->esp? wm->esp : "", &ugh, FALSE);
-		} else if(c->policy & POLICY_AUTHENTICATE) {
-		    c->alg_info_esp = alg_info_ah_create_from_str(wm->esp? wm->esp : "", &ugh, FALSE);
-		} else {
+		if( (c->policy & POLICY_AUTHENTICATE)
+		    && (c->policy & POLICY_ENCRYPT)) {
 		    loglog(RC_NOALGO, "Can only do AH, or ESP, not AH+ESP\n");
 		    return;
 		}
+		if( !(c->policy & POLICY_AUTHENTICATE)
+		    && !(c->policy & POLICY_ENCRYPT)) {
+		    loglog(RC_NOALGO, "Must do at AH or ESP, not neither.\n");
+		    return;
+		}
+
+		if(c->policy & POLICY_ENCRYPT) {
+		    c->alg_info_esp = alg_info_esp_create_from_str(wm->esp? wm->esp : "", &ugh, FALSE);
+		} 
+
+		if(c->policy & POLICY_AUTHENTICATE) {
+		    c->alg_info_esp = alg_info_ah_create_from_str(wm->esp? wm->esp : "", &ugh, FALSE);
+		} 
 
 		DBG(DBG_CRYPT|DBG_CONTROL, 
 			static char buf[256]="<NULL>";
@@ -2197,6 +2210,8 @@ refine_host_connection(const struct state *st, const struct id *peer_id
 
     psk = NULL;
 
+    zero(&peer_ca);
+
     our_pathlen = peer_pathlen = 0;
     best_our_pathlen  = 0;
     best_peer_pathlen = 0;
@@ -2436,7 +2451,7 @@ is_virtual_net_used(struct connection *c, const ip_subnet *peer_net, const struc
 				 , enum_name(&connection_kind_names, d->kind)
 				 , buf);
 
-		    if(!kernel_overlap_supported()) {
+		    if(!kernel_ops->overlap_supported) {
 			openswan_log("Kernel method '%s' does not support overlapping IP ranges"
 				     , kernel_if_name());
 			return TRUE;
@@ -2690,8 +2705,8 @@ fc_try_oppo(const struct connection *c
 	    continue;
 
 	if (!(same_id(&c->spd.this.id, &d->spd.this.id)
-	      && match_id(&c->spd.that.id, &d->spd.that.id, &wildcards)
-	      && trusted_ca(c->spd.that.ca, d->spd.that.ca, &pathlen)))
+	&& match_id(&c->spd.that.id, &d->spd.that.id, &wildcards)
+	&& trusted_ca(c->spd.that.ca, d->spd.that.ca, &pathlen)))
 	    continue;
 
 	/* compare protocol and ports */
@@ -2936,225 +2951,11 @@ connection_compare_qsort(const void *a, const void *b)
 			    , *(const struct connection *const *)b);
 }
 
-static void
-show_one_sr(struct connection *c
-	    , struct spd_route *sr
-	    , int num UNUSED
-	    , char *instance)
-{
-    char topo[CONN_BUF_LEN];
-    char srcip[ADDRTOT_BUF], dstip[ADDRTOT_BUF];
-    char thissemi[3+sizeof("myup=")];
-    char thatsemi[3+sizeof("hisup=")];
-#ifdef XAUTH
-    char thisxauthsemi[XAUTH_USERNAME_LEN+sizeof("myxauthuser=")];
-    char thatxauthsemi[XAUTH_USERNAME_LEN+sizeof("hisxauthuser=")];
-#endif
-    char thiscertsemi[3+sizeof("mycert=")+PATH_MAX];
-    char thatcertsemi[3+sizeof("hiscert=")+PATH_MAX];
-    char *thisup, *thatup;
-    
-    (void) format_connection(topo, sizeof(topo), c, sr);
-    whack_log(RC_COMMENT, "\"%s\"%s: %s; %s; eroute owner: #%lu"
-	      , c->name, instance, topo
-	      , enum_name(&routing_story, sr->routing)
-	      , sr->eroute_owner);
-    if(addrbytesptr(&c->spd.this.host_srcip, NULL) == 0
-       || isanyaddr(&c->spd.this.host_srcip)) {
-	strcpy(srcip, "unset");
-    } else {
-	addrtot(&sr->this.host_srcip, 0, srcip, sizeof(srcip));
-    }
-    if(addrbytesptr(&c->spd.that.host_srcip, NULL) == 0
-       || isanyaddr(&c->spd.that.host_srcip)) {
-	strcpy(dstip, "unset");
-    } else {
-	addrtot(&sr->that.host_srcip, 0, dstip, sizeof(dstip));
-    }
-    
-    thissemi[0]='\0';
-    thisup=thissemi;
-    if(sr->this.updown) {
-	thissemi[0]=';';
-	thissemi[1]=' ';
-	thissemi[2]='\0';
-	strcat(thissemi, "myup=");
-	thisup=sr->this.updown;
-    }
-    
-    thatsemi[0]='\0';
-    thatup=thatsemi;
-    if(sr->that.updown) {
-	thatsemi[0]=';';
-	thatsemi[1]=' ';
-	thatsemi[2]='\0';
-	strcat(thatsemi, "hisup=");
-	thatup=sr->that.updown;
-    }
-    
-    thiscertsemi[0]='\0';
-    if(sr->this.cert_filename) {
-	snprintf(thiscertsemi, sizeof(thiscertsemi)-1
-		 , "; mycert=%s"
-		 , sr->this.cert_filename);
-    }
-    
-    thatcertsemi[0]='\0';
-    if(sr->that.cert_filename) {
-	snprintf(thatcertsemi, sizeof(thatcertsemi)-1
-		 , "; hiscert=%s"
-		 , sr->that.cert_filename);
-    }
-
-    whack_log(RC_COMMENT, "\"%s\"%s:     myip=%s; hisip=%s%s%s%s%s%s%s;"
-	      , c->name, instance, srcip, dstip
-	      , thissemi, thisup
-	      , thatsemi, thatup
-	      , thiscertsemi
-	      , thatcertsemi);
-    
-#ifdef XAUTH
-    if(sr->this.xauth_name || sr->that.xauth_name) {
-	thisxauthsemi[0]='\0';
-	if(sr->this.xauth_name) {
-	    snprintf(thisxauthsemi, sizeof(thisxauthsemi)-1
-		     , "myxauthuser=%s; "
-		     , sr->this.xauth_name);
-	}
-	
-	thatxauthsemi[0]='\0';
-	if(sr->that.xauth_name) {
-	    snprintf(thatxauthsemi, sizeof(thatxauthsemi)-1
-		     , "hisxauthuser=%s; "
-		     , sr->that.xauth_name);
-	}
-	whack_log(RC_COMMENT, "\"%s\"%s:     xauth info: %s%s"
-		  , c->name, instance
-		  , thisxauthsemi
-		  , thatxauthsemi);
-    }
-#endif
-}
-
-void 
-show_one_connection(struct connection *c)
-{
-    const char *ifn;
-    char instance[1 + 10 + 1];
-    char prio[POLICY_PRIO_BUF];
-    
-    ifn = oriented(*c)? c->interface->ip_dev->id_rname : "";
-
-    instance[0] = '\0';
-    if (c->kind == CK_INSTANCE && c->instance_serial != 0)
-	snprintf(instance, sizeof(instance), "[%lu]", c->instance_serial);
-    
-    /* show topology */
-    {
-	struct spd_route *sr = &c->spd;
-	int num=0;
-	
-	while (sr != NULL)
-	{
-	    show_one_sr(c, sr, num, instance);
-	    sr = sr->next;
-	    num++;
-	}
-    }
-    
-    /* show CAs */
-    if (c->spd.this.ca.ptr != NULL || c->spd.that.ca.ptr != NULL)
-    {
-	char this_ca[IDTOA_BUF], that_ca[IDTOA_BUF];
-	
-	dntoa_or_null(this_ca, IDTOA_BUF, c->spd.this.ca, "%any");
-	dntoa_or_null(that_ca, IDTOA_BUF, c->spd.that.ca, "%any");
-	
-	whack_log(RC_COMMENT
-		  , "\"%s\"%s:   CAs: '%s'...'%s'"
-		  , c->name
-		  , instance
-		  , this_ca
-		  , that_ca);
-    }
-    
-    whack_log(RC_COMMENT
-	      , "\"%s\"%s:   ike_life: %lus; ipsec_life: %lus;"
-	      " rekey_margin: %lus; rekey_fuzz: %lu%%; keyingtries: %lu"
-	      , c->name
-	      , instance
-	      , (unsigned long) c->sa_ike_life_seconds
-	      , (unsigned long) c->sa_ipsec_life_seconds
-	      , (unsigned long) c->sa_rekey_margin
-	      , (unsigned long) c->sa_rekey_fuzz
-	      , (unsigned long) c->sa_keying_tries);
-
-    if (c->policy_next)
-    {
-	whack_log(RC_COMMENT
-		  , "\"%s\"%s:   policy_next: %s"
-		  , c->name, instance, c->policy_next->name);
-    }
-    
-    /* Note: we display key_from_DNS_on_demand as if policy [lr]KOD */
-    fmt_policy_prio(c->prio, prio);
-    whack_log(RC_COMMENT
-	      , "\"%s\"%s:   policy: %s%s%s; prio: %s; interface: %s; "
-	      , c->name
-	      , instance
-	      , prettypolicy(c->policy)
-	      , c->spd.this.key_from_DNS_on_demand? "+lKOD" : ""
-	      , c->spd.that.key_from_DNS_on_demand? "+rKOD" : ""
-	      , prio
-	      , ifn);
-    
-    /* slightly complicated stuff to avoid extra crap */
-    if(c->dpd_timeout > 0 || DBGP(DBG_DPD)) {
-	whack_log(RC_COMMENT
-		  , "\"%s\"%s:   dpd: %s; delay:%lu; timeout:%lu; "
-		  , c->name
-		  , instance
-		  , enum_name(&dpd_action_names, c->dpd_action)
-		  , (unsigned long)c->dpd_delay
-		  , (unsigned long)c->dpd_timeout);
-    }
-
-    if(c->extra_debugging) {
-	whack_log(RC_COMMENT, "\"%s\"%s:   debug: %s"
-		  , c->name
-		  , instance
-		  , bitnamesof(debug_bit_names
-			       , c->extra_debugging));
-    }
-    
-    whack_log(RC_COMMENT
-	      , "\"%s\"%s:   newest ISAKMP SA: #%ld; newest IPsec SA: #%ld; "
-	      , c->name
-	      , instance
-	      , c->newest_isakmp_sa
-	      , c->newest_ipsec_sa);
-    
-    if(c->connalias) {
-	whack_log(RC_COMMENT
-		  , "\"%s\"%s:   aliases: %s\n"
-		  , c->name
-		  , instance
-		  , c->connalias);
-    }
-    
-#ifdef IKE_ALG
-    ike_alg_show_connection(c, instance);
-#endif
-#ifdef KERNEL_ALG
-    kernel_alg_show_connection(c, instance);
-#endif
-}
-
 void
 show_connections_status(void)
 {
-    int count, i;
     struct connection *c;
+    int count, i;
     struct connection **array;
 
 
@@ -3164,6 +2965,10 @@ show_connections_status(void)
     {
 	count++;
     }
+    if(count == 0) 
+	/* abort early to avoid a malloc(0) that uclibc does not like */
+	return;
+
     array = alloc_bytes(sizeof(struct connection *)*count, "connection array");
 
     count=0;
@@ -3177,7 +2982,208 @@ show_connections_status(void)
 
     for (i=0; i<count; i++)
     {
-	show_one_connection(array[i]);
+	const char *ifn;
+	char instance[1 + 10 + 1];
+	char prio[POLICY_PRIO_BUF];
+
+	c = array[i];
+
+	ifn = oriented(*c)? c->interface->ip_dev->id_rname : "";
+
+	instance[0] = '\0';
+	if (c->kind == CK_INSTANCE && c->instance_serial != 0)
+	    snprintf(instance, sizeof(instance), "[%lu]", c->instance_serial);
+
+	/* show topology */
+	{
+	    char topo[CONN_BUF_LEN];
+	    struct spd_route *sr = &c->spd;
+	    int num=0;
+
+	    while (sr != NULL)
+	    {
+		char srcip[ADDRTOT_BUF], dstip[ADDRTOT_BUF];
+		char thissemi[3+sizeof("myup=")];
+		char thatsemi[3+sizeof("hisup=")];
+#ifdef XAUTH
+		char thisxauthsemi[XAUTH_USERNAME_LEN+sizeof("myxauthuser=")];
+		char thatxauthsemi[XAUTH_USERNAME_LEN+sizeof("hisxauthuser=")];
+#endif
+		char thiscertsemi[3+sizeof("mycert=")+PATH_MAX];
+		char thatcertsemi[3+sizeof("hiscert=")+PATH_MAX];
+		char *thisup, *thatup;
+
+		(void) format_connection(topo, sizeof(topo), c, sr);
+		whack_log(RC_COMMENT, "\"%s\"%s: %s; %s; eroute owner: #%lu"
+			  , c->name, instance, topo
+			  , enum_name(&routing_story, sr->routing)
+			  , sr->eroute_owner);
+		if(addrbytesptr(&c->spd.this.host_srcip, NULL) == 0
+		   || isanyaddr(&c->spd.this.host_srcip)) {
+		    strcpy(srcip, "unset");
+		} else {
+		    addrtot(&sr->this.host_srcip, 0, srcip, sizeof(srcip));
+		}
+		if(addrbytesptr(&c->spd.that.host_srcip, NULL) == 0
+		   || isanyaddr(&c->spd.that.host_srcip)) {
+		    strcpy(dstip, "unset");
+		} else {
+		    addrtot(&sr->that.host_srcip, 0, dstip, sizeof(dstip));
+		}
+
+		thissemi[0]='\0';
+		thisup=thissemi;
+		if(sr->this.updown) {
+		    thissemi[0]=';';
+		    thissemi[1]=' ';
+		    thissemi[2]='\0';
+		    strcat(thissemi, "myup=");
+		    thisup=sr->this.updown;
+		}
+		
+		thatsemi[0]='\0';
+		thatup=thatsemi;
+		if(sr->that.updown) {
+		    thatsemi[0]=';';
+		    thatsemi[1]=' ';
+		    thatsemi[2]='\0';
+		    strcat(thatsemi, "hisup=");
+		    thatup=sr->that.updown;
+		}
+		
+		thiscertsemi[0]='\0';
+		if(sr->this.cert_filename) {
+		    snprintf(thiscertsemi, sizeof(thiscertsemi)-1
+			     , "; mycert=%s"
+			     , sr->this.cert_filename);
+		}
+
+		thatcertsemi[0]='\0';
+		if(sr->that.cert_filename) {
+		    snprintf(thatcertsemi, sizeof(thatcertsemi)-1
+			     , "; hiscert=%s"
+			     , sr->that.cert_filename);
+		}
+
+		whack_log(RC_COMMENT, "\"%s\"%s:     myip=%s; hisip=%s%s%s%s%s%s%s;"
+			  , c->name, instance, srcip, dstip
+			  , thissemi, thisup
+			  , thatsemi, thatup
+			  , thiscertsemi
+			  , thatcertsemi);
+
+#ifdef XAUTH
+		if(sr->this.xauth_name || sr->that.xauth_name) {
+		    thisxauthsemi[0]='\0';
+		    if(sr->this.xauth_name) {
+			snprintf(thisxauthsemi, sizeof(thisxauthsemi)-1
+				 , "myxauthuser=%s; "
+				 , sr->this.xauth_name);
+		    }
+		    
+		    thatxauthsemi[0]='\0';
+		    if(sr->that.xauth_name) {
+			snprintf(thatxauthsemi, sizeof(thatxauthsemi)-1
+				 , "hisxauthuser=%s; "
+				 , sr->that.xauth_name);
+		    }
+		    whack_log(RC_COMMENT, "\"%s\"%s:     xauth info: %s%s"
+			      , c->name, instance
+			      , thisxauthsemi
+			      , thatxauthsemi);
+		}
+#endif
+		sr = sr->next;
+		num++;
+	    }
+	}
+
+	/* show CAs */
+	if (c->spd.this.ca.ptr != NULL || c->spd.that.ca.ptr != NULL)
+	{
+	    char this_ca[IDTOA_BUF], that_ca[IDTOA_BUF];
+
+	    dntoa_or_null(this_ca, IDTOA_BUF, c->spd.this.ca, "%any");
+	    dntoa_or_null(that_ca, IDTOA_BUF, c->spd.that.ca, "%any");
+
+	    whack_log(RC_COMMENT
+		, "\"%s\"%s:   CAs: '%s'...'%s'"
+		, c->name
+		, instance
+		, this_ca
+		, that_ca);
+	}
+
+	whack_log(RC_COMMENT
+	    , "\"%s\"%s:   ike_life: %lus; ipsec_life: %lus;"
+	    " rekey_margin: %lus; rekey_fuzz: %lu%%; keyingtries: %lu"
+	    , c->name
+	    , instance
+	    , (unsigned long) c->sa_ike_life_seconds
+	    , (unsigned long) c->sa_ipsec_life_seconds
+	    , (unsigned long) c->sa_rekey_margin
+	    , (unsigned long) c->sa_rekey_fuzz
+	    , (unsigned long) c->sa_keying_tries);
+
+	if (c->policy_next)
+	{
+	    whack_log(RC_COMMENT
+		      , "\"%s\"%s:   policy_next: %s"
+		      , c->name, instance, c->policy_next->name);
+	}
+
+	/* Note: we display key_from_DNS_on_demand as if policy [lr]KOD */
+	fmt_policy_prio(c->prio, prio);
+	whack_log(RC_COMMENT
+	    , "\"%s\"%s:   policy: %s%s%s; prio: %s; interface: %s; "
+	    , c->name
+	    , instance
+	    , prettypolicy(c->policy)
+	    , c->spd.this.key_from_DNS_on_demand? "+lKOD" : ""
+	    , c->spd.that.key_from_DNS_on_demand? "+rKOD" : ""
+	    , prio
+	    , ifn);
+
+	/* slightly complicated stuff to avoid extra crap */
+	if(c->dpd_timeout > 0 || DBGP(DBG_DPD)) {
+	    whack_log(RC_COMMENT
+		      , "\"%s\"%s:   dpd: %s; delay:%lu; timeout:%lu; "
+		      , c->name
+		      , instance
+		      , enum_name(&dpd_action_names, c->dpd_action)
+		      , (unsigned long)c->dpd_delay
+		      , (unsigned long)c->dpd_timeout);
+	}
+
+	if(c->extra_debugging) {
+	    whack_log(RC_COMMENT, "\"%s\"%s:   debug: %s"
+		      , c->name
+		      , instance
+		      , bitnamesof(debug_bit_names
+				   , c->extra_debugging));
+	}
+
+	whack_log(RC_COMMENT
+	    , "\"%s\"%s:   newest ISAKMP SA: #%ld; newest IPsec SA: #%ld; "
+	    , c->name
+	    , instance
+	    , c->newest_isakmp_sa
+	    , c->newest_ipsec_sa);
+
+	if(c->connalias) {
+	    whack_log(RC_COMMENT
+		      , "\"%s\"%s:   aliases: %s\n"
+		      , c->name
+		      , instance
+		      , c->connalias);
+	}
+
+#ifdef IKE_ALG
+	ike_alg_show_connection(c, instance);
+#endif
+#ifdef KERNEL_ALG
+	kernel_alg_show_connection(c, instance);
+#endif
     }
     pfree(array);
 }
