@@ -126,35 +126,34 @@ stf_status build_ke(struct pluto_crypto_req_cont *cn
 
     memset(&rd, 0, sizeof(rd));
 
-  r->pcr_len  = sizeof(struct pluto_crypto_req);
-  r->pcr_type = pcr_build_kenonce;
-  r->pcr_pcim = importance;
+    r->pcr_len  = sizeof(struct pluto_crypto_req);
+    r->pcr_type = pcr_build_kenonce;
+    r->pcr_pcim = importance;
 
-  r->pcr_d.kn.thespace.start = 0;
-  r->pcr_d.kn.thespace.len   = sizeof(r->pcr_d.kn.space);
-  r->pcr_d.kn.oakley_group   = group->group;
-
-  cn->pcrc_serialno = st->st_serialno;
-  e= send_crypto_helper_request(r, cn, &toomuch);
-
-  if(e != NULL) {
-      loglog(RC_LOG_SERIOUS, "can not start crypto helper: %s", e);
-      if(toomuch) {
-	  return STF_TOOMUCHCRYPTO;
-      } else {
-	  return STF_FAIL;
-      }
-  } else if(!toomuch) {
-      st->st_calculating = TRUE;
-      delete_event(st);
-      event_schedule(EVENT_CRYPTO_FAILED, EVENT_CRYPTO_FAILED_DELAY, st);
-      return STF_SUSPEND;
-  } else {
-      /* we must have run the continuation directly, so
-       * complete_state_transition already got called. 
-       */
-      return STF_INLINE;
-  }
+    pcr_init(r);
+    r->pcr_d.kn.oakley_group   = group->group;
+    
+    cn->pcrc_serialno = st->st_serialno;
+    e= send_crypto_helper_request(r, cn, &toomuch);
+    
+    if(e != NULL) {
+	loglog(RC_LOG_SERIOUS, "can not start crypto helper: %s", e);
+	if(toomuch) {
+	    return STF_TOOMUCHCRYPTO;
+	} else {
+	    return STF_FAIL;
+	}
+    } else if(!toomuch) {
+	st->st_calculating = TRUE;
+	delete_event(st);
+	event_schedule(EVENT_CRYPTO_FAILED, EVENT_CRYPTO_FAILED_DELAY, st);
+	return STF_SUSPEND;
+    } else {
+	/* we must have run the continuation directly, so
+	 * complete_v1_state_transition already got called. 
+	 */
+	return STF_INLINE;
+    }
 }
 
 
@@ -193,7 +192,7 @@ stf_status build_nonce(struct pluto_crypto_req_cont *cn
       return STF_SUSPEND;
   } else {
       /* we must have run the continuation directly, so
-       * complete_state_transition already got called. 
+       * complete_v1_state_transition already got called. 
        */
       return STF_INLINE;
   }
@@ -202,7 +201,7 @@ stf_status build_nonce(struct pluto_crypto_req_cont *cn
 
 /*
  * Local Variables:
- * c-basic-offset:4
+ * c-basic-offset: 4
  * c-style: pluto
  * End:
  */
