@@ -196,7 +196,7 @@ compute_proto_keymat(struct state *st
 		     , struct ipsec_proto_info *pi
 		     , const char *satypename)
 {
-    size_t needed_len; /* bytes of keying material needed */
+    size_t needed_len = 0; /* bytes of keying material needed */
 
     /* Add up the requirements for keying material
      * (It probably doesn't matter if we produce too much!)
@@ -217,7 +217,6 @@ compute_proto_keymat(struct state *st
 		break;
 	    case ESP_AES:
 		needed_len = AES_CBC_BLOCK_SIZE;
-
 		/* if an attribute is set, then use that! */
 		if(st->st_esp.attrs.transattrs.enckeylen) {
 		    needed_len = st->st_esp.attrs.transattrs.enckeylen/8;
@@ -545,11 +544,11 @@ decode_net_id(struct isakmp_ipsec_id *id
 /* like decode, but checks that what is received matches what was sent */
 static bool
 check_net_id(struct isakmp_ipsec_id *id
-, pb_stream *id_pbs
-, u_int8_t *protoid
-, u_int16_t *port
-, ip_subnet *net
-, const char *which)
+	     , pb_stream *id_pbs
+	     , u_int8_t *protoid
+	     , u_int16_t *port
+	     , ip_subnet *net
+	     , const char *which)
 {
     ip_subnet net_temp;
     bool bad_proposal=FALSE;
@@ -1666,7 +1665,10 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 		, buf);
 	    return STF_FAIL + INVALID_ID_INFORMATION;
 	}
-	else if (p != c)
+
+	/* did we find a better connection? */
+	/* should we use an else here, as we did in osw 2.5.x? */
+	if (p != c)
 	{
 	    /* We've got a better connection: it can support the
 	     * specified clients.  But it may need instantiation.
@@ -1752,11 +1754,19 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 #endif
 	    c = p;
 	}
+
+	/* XXX Though c == p, they are used intermixed in the below section */
 	/* fill in the client's true ip address/subnet */
-	if (p->spd.that.has_client_wildcard)
+	DBG(DBG_CONTROLMORE
+	    , DBG_log("client wildcard: %s  port wildcard: %s  virtual: %s"
+		      , c->spd.that.has_client_wildcard ? "yes" : "no"
+		      , c->spd.that.has_port_wildcard  ? "yes" : "no"
+		      , is_virtual_connection(c) ? "yes" : "no"));
+
+	if (c->spd.that.has_client_wildcard)
 	{
-	    p->spd.that.client = *his_net;
-	    p->spd.that.has_client_wildcard = FALSE;
+	    c->spd.that.client = *his_net;
+	    c->spd.that.has_client_wildcard = FALSE;
 	}
 
         /* fill in the client's true port */
@@ -1775,10 +1785,22 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 
 	else if (is_virtual_connection(c))
 	{
+	    char cthat[END_BUF];
+
 	    c->spd.that.client = *his_net;
+	    c->spd.that.has_client = TRUE;
 	    c->spd.that.virt = NULL;
-	    if (subnetishost(his_net) && addrinsubnet(&c->spd.that.host_addr, his_net))
+
+	    if (subnetishost(his_net)
+		&& addrinsubnet(&c->spd.that.host_addr, his_net)) {
+
 		c->spd.that.has_client = FALSE;
+	    }
+
+	    format_end(cthat, sizeof(cthat), &c->spd.that, NULL, TRUE, LEMPTY);
+	    DBG(DBG_CONTROLMORE
+		, DBG_log("setting phase 2 virtual values to %s"
+			  , cthat));
 	}
     }
     passert((p1st->st_policy & POLICY_PFS)==0 || p1st->st_pfs_group != NULL );
@@ -2068,8 +2090,18 @@ quick_inI1_outR1_cryptotail(struct dh_continuation *dh
 	loglog(RC_LOG_SERIOUS, "we require PFS but Quick I1 SA specifies no GROUP_DESCRIPTION");
 	return STF_FAIL + NO_PROPOSAL_CHOSEN;	/* ??? */
     }
-    
-    openswan_log("responding to Quick Mode {msgid:%08x}", st->st_msgid);
+
+    openswan_log("responding to Quick Mode proposal {msgid:%08x}", st->st_msgid);
+    {
+	char instbuf[END_BUF];
+	struct connection *c = st->st_connection;
+	struct spd_route *sr = &c->spd;
+
+	format_end(instbuf, sizeof(instbuf),&sr->this,&sr->that,TRUE, LEMPTY);
+	openswan_log("    us: %s", instbuf);
+	
+	format_end(instbuf, sizeof(instbuf),&sr->that,&sr->this,FALSE, LEMPTY);	openswan_log("  them: %s", instbuf);
+    }
 
     /**** finish reply packet: Nr [, KE ] [, IDci, IDcr ] ****/
     
@@ -2325,11 +2357,10 @@ quick_inR1_outI2_cryptotail(struct dh_continuation *dh
 	    /* ??? we are assuming IPSEC_DOI */
 
 	    /* IDci (we are initiator) */
-
 	    if (!check_net_id(&IDci->payload.ipsec_id, &IDci->pbs
-	    , &st->st_myuserprotoid, &st->st_myuserport
-	    , &st->st_connection->spd.this.client
-	    , "our client"))
+			      , &st->st_myuserprotoid, &st->st_myuserport
+			      , &st->st_connection->spd.this.client
+			      , "our client"))
 		return STF_FAIL + INVALID_ID_INFORMATION;
 
 	    /* we checked elsewhere that we got two of them */
@@ -2339,9 +2370,9 @@ quick_inR1_outI2_cryptotail(struct dh_continuation *dh
 	    /* IDcr (responder is peer) */
 
 	    if (!check_net_id(&IDcr->payload.ipsec_id, &IDcr->pbs
-	    , &st->st_peeruserprotoid, &st->st_peeruserport
-	    , &st->st_connection->spd.that.client
-	    , "peer client"))
+			      , &st->st_peeruserprotoid, &st->st_peeruserport
+			      , &st->st_connection->spd.that.client
+			      , "peer client"))
 		return STF_FAIL + INVALID_ID_INFORMATION;
 
 	    /*
