@@ -80,7 +80,9 @@ struct kernel_sa {
 	unsigned enckeylen;
 	unsigned char *enckey;
 
-	IPsecSAref_t ref, refhim;
+	int outif;
+	IPsecSAref_t ref;
+	IPsecSAref_t refhim;
 
 	int encapsulation;
 #ifdef NAT_TRAVERSAL
@@ -137,12 +139,12 @@ struct kernel_ops {
 			 , enum pluto_sadb_operations op
 			 , const char *opname);
     bool (*sag_eroute)(struct state *st, struct spd_route *sr
-		       , unsigned op, const char *opname);
+		       , enum pluto_sadb_operations op, const char *opname);
     bool (*eroute_idle)(struct state *st, time_t idle_max);
     void (*remove_orphaned_holds)(int transportproto
 				  , const ip_subnet *ours
 				  , const ip_subnet *his);
-    bool (*add_sa)(const struct kernel_sa *sa, bool replace);
+    bool (*add_sa)(struct kernel_sa *sa, bool replace);
     bool (*grp_sa)(const struct kernel_sa *sa_outer,
 		   const struct kernel_sa *sa_inner);
     bool (*del_sa)(const struct kernel_sa *sa);
@@ -167,6 +169,7 @@ extern int create_socket(struct raw_iface *ifp, const char *v_name, int port);
 # define IPSECDEVPREFIX "ipsec"
 #endif
 
+extern int useful_mastno;
 #ifndef MASTDEVPREFIX
 # define MASTDEVPREFIX  "mast"
 #endif
@@ -196,6 +199,12 @@ extern bool do_command_freebsd(struct connection *c, struct spd_route *sr
 extern bool invoke_command(const char *verb, const char *verb_suffix, char *cmd);
 #endif
 
+#if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
+extern bool do_command_darwin(struct connection *c, struct spd_route *sr
+			       , const char *verb, struct state *st);
+extern bool invoke_command(const char *verb, const char *verb_suffix, char *cmd);
+#endif
+
 #if defined(__CYGWIN32__)
 extern bool do_command_cygwin(struct connection *c, struct spd_route *sr
 			      , const char *verb, struct state *st);
@@ -327,7 +336,6 @@ extern bool eroute_connection(struct spd_route *sr
 extern const struct kernel_ops klips_kernel_ops;
 extern const struct kernel_ops mast_kernel_ops;
 
-extern bool kernel_overlap_supported(void);
 extern const char *kernel_if_name(void);
 extern void show_kernel_interface(void);
 
