@@ -23,6 +23,8 @@
 #include <string.h>
 #include <sys/socket.h>
 #include <sys/types.h>
+#include <stdint.h>
+#include <linux/pfkeyv2.h>
 #include <unistd.h>
 
 #include "kameipsec.h"
@@ -43,19 +45,44 @@
 #include "kernel.h"
 #include "server.h"
 #include "nat_traversal.h"
+#include "state.h"
 #include "kernel_netlink.h"
 #include "kernel_pfkey.h"
 #include "log.h"
 #include "whack.h"	/* for RC_LOG_SERIOUS */
 #include "kernel_alg.h"
+#include "klips-crypto/aes_cbc.h"
+#include "ike_alg.h"
 
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
 
+const struct pfkey_proto_info null_proto_info[2];
+
+static const struct pfkey_proto_info broad_proto_info[2] = { 
+        {
+                proto: IPPROTO_ESP,
+                encapsulation: ENCAPSULATION_MODE_TUNNEL,
+                reqid: 0
+        },
+        {
+                proto: 0,
+                encapsulation: 0,
+                reqid: 0
+        }
+};
+
+
 /* Minimum priority number in SPD used by pluto. */
 #define MIN_SPD_PRIORITY 1024
 
+struct aead_alg {
+	int id;
+	int icvlen;
+	const char *name;
+};
+
 static int netlinkfd = NULL_FD;
 static int netlink_bcast_fd = NULL_FD;
 
@@ -109,6 +136,8 @@ static sparse_names ealg_list = {
 	{ SADB_X_EALG_CASTCBC, "cast128" },
 	{ SADB_X_EALG_BLOWFISHCBC, "blowfish" },
 	{ SADB_X_EALG_AESCBC, "aes" },
+	{ SADB_X_EALG_AESCTR, "ctr(aes)" },
+	{ SADB_X_EALG_CAMELLIACBC, "cbc(camellia)" },
 	{ 0, sparse_end }
 };
 
@@ -120,6 +149,26 @@ static sparse_names calg_list = {
 	{ 0, sparse_end }
 };
 
+static struct aead_alg aead_algs[] =
+{
+	{ .id = SADB_X_EALG_AES_CCM_ICV8, .icvlen = 8, .name = "rfc4309(ccm(aes))" },
+	{ .id = SADB_X_EALG_AES_CCM_ICV12, .icvlen = 12, .name = "rfc4309(ccm(aes))" },
+	{ .id = SADB_X_EALG_AES_CCM_ICV16, .icvlen = 16, .name = "rfc4309(ccm(aes))" },
+	{ .id = SADB_X_EALG_AES_GCM_ICV8, .icvlen = 8, .name = "rfc4106(gcm(aes))" },
+	{ .id = SADB_X_EALG_AES_GCM_ICV12, .icvlen = 12, .name = "rfc4106(gcm(aes))" },
+	{ .id = SADB_X_EALG_AES_GCM_ICV16, .icvlen = 16, .name = "rfc4106(gcm(aes))" },
+};
+
+static struct aead_alg *get_aead_alg(int algid)
+{
+    unsigned int i;
+
+    for (i = 0; i < sizeof(aead_algs) / sizeof(aead_algs[0]); i++)
+	if (aead_algs[i].id == algid)
+		return aead_algs + i;
+    return NULL;
+}
+
 /** ip2xfrm - Take an IP address and convert to an xfrm.
  *
  * @param addr ip_address
@@ -415,7 +464,7 @@ netlink_raw_eroute(const ip_address *this_host
 
     policy = IPSEC_POLICY_IPSEC;
 
-    if (satype == SADB_X_SATYPE_INT)
+    if (satype == K_SADB_X_SATYPE_INT)
     {
 	/* shunt route */
 	switch (ntohl(spi))
@@ -497,7 +546,13 @@ netlink_raw_eroute(const ip_address *this_host
 	req.u.p.lft.hard_byte_limit = XFRM_INF;
 	req.u.p.lft.hard_packet_limit = XFRM_INF;
 
-	req.n.nlmsg_type = XFRM_MSG_NEWPOLICY;
+	/*
+	 * NEW will fail when an existing policy, UPD always works.
+	 * This seems to happen in cases with NAT'ed XP clients, or
+	 * quick recycling/resurfacing of roadwarriors on the same IP.
+	 * req.n.nlmsg_type = XFRM_MSG_NEWPOLICY;
+	 */
+	req.n.nlmsg_type = XFRM_MSG_UPDPOLICY;
 	if (sadb_op == ERO_REPLACE)
 	{
 	    req.n.nlmsg_type = XFRM_MSG_UPDPOLICY;
@@ -562,7 +617,7 @@ netlink_raw_eroute(const ip_address *this_host
 	    break;
 	}
 	else if (proto_info[0].encapsulation != ENCAPSULATION_MODE_TUNNEL
-	&& satype != SADB_X_SATYPE_INT)
+		 && satype != K_SADB_X_SATYPE_INT)
 	{
 	    break;
 	}
@@ -584,7 +639,7 @@ netlink_raw_eroute(const ip_address *this_host
  * @return bool True if successfull
  */
 static bool
-netlink_add_sa(const struct kernel_sa *sa, bool replace)
+netlink_add_sa(struct kernel_sa *sa, bool replace)
 {
     struct {
 	struct nlmsghdr n;
@@ -592,6 +647,7 @@ netlink_add_sa(const struct kernel_sa *sa, bool replace)
 	char data[1024];
     } req;
     struct rtattr *attr;
+    struct aead_alg *aead;
 
     memset(&req, 0, sizeof(req));
     req.n.nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK;
@@ -641,7 +697,26 @@ netlink_add_sa(const struct kernel_sa *sa, bool replace)
 	attr = (struct rtattr *)((char *)attr + attr->rta_len);
     }
 
-    if (sa->enckeylen)
+    aead = get_aead_alg(sa->encalg);
+    if (aead)
+    {
+	struct xfrm_algo_aead algo;
+
+	strcpy(algo.alg_name, aead->name);
+	algo.alg_key_len = sa->enckeylen * BITS_PER_BYTE;
+	algo.alg_icv_len = aead->icvlen * BITS_PER_BYTE;
+
+	attr->rta_type = XFRMA_ALG_AEAD;
+	attr->rta_len = RTA_LENGTH(sizeof(algo) + sa->enckeylen);
+
+	memcpy(RTA_DATA(attr), &algo, sizeof(algo));
+	memcpy((char *)RTA_DATA(attr) + sizeof(algo), sa->enckey
+	    , sa->enckeylen);
+
+	req.n.nlmsg_len += attr->rta_len;
+	attr = (struct rtattr *)((char *)attr + attr->rta_len);
+    }
+    else
     {
 	struct xfrm_algo algo;
 	const char *name;
@@ -667,7 +742,7 @@ netlink_add_sa(const struct kernel_sa *sa, bool replace)
 	attr = (struct rtattr *)((char *)attr + attr->rta_len);
     }
 
-    if (sa->satype == SADB_X_SATYPE_COMP)
+    if (sa->satype == SADB_X_SATYPE_IPCOMP)
     {
 	struct xfrm_algo algo;
 	const char *name;
@@ -743,6 +818,125 @@ netlink_del_sa(const struct kernel_sa *sa)
     return send_netlink_msg(&req.n, NULL, 0, "Del SA", sa->text_said);
 }
 
+/* XXX Move these elsewhere */
+#define  AES_KEY_MIN_LEN       128
+#define  AES_KEY_DEF_LEN       128
+#define  AES_KEY_MAX_LEN       256
+
+struct encrypt_desc algo_aes_ccm_8 =
+{
+	common: {
+	  name: "aes_ccm_8",
+	  officname: "aes_ccm_8",
+	  algo_type:    IKE_ALG_ENCRYPT,
+	  algo_v2id:    IKEv2_ENCR_AES_CCM_8,
+	  algo_next:    NULL, },
+	enc_blocksize:  AES_CBC_BLOCK_SIZE,
+	keyminlen:      AES_KEY_MIN_LEN + 3,
+	keydeflen:      AES_KEY_DEF_LEN + 3,
+	keymaxlen:      AES_KEY_MAX_LEN + 3,
+};
+
+struct encrypt_desc algo_aes_ccm_12 =
+{
+	common: {
+	  name: "aes_ccm_12",
+	  officname: "aes_ccm_12",
+	  algo_type:    IKE_ALG_ENCRYPT,
+	  algo_v2id:    IKEv2_ENCR_AES_CCM_12,
+	  algo_next:    NULL, },
+	enc_blocksize:  AES_CBC_BLOCK_SIZE,
+	keyminlen:      AES_KEY_MIN_LEN + 3,
+	keydeflen:      AES_KEY_DEF_LEN + 3,
+	keymaxlen:      AES_KEY_MAX_LEN + 3,
+};
+
+struct encrypt_desc algo_aes_ccm_16 =
+{
+	common: {
+	  name: "aes_ccm_16",
+	  officname: "aes_ccm_16",
+	  algo_type: 	IKE_ALG_ENCRYPT,
+	  algo_v2id:    IKEv2_ENCR_AES_CCM_16,
+	  algo_next: 	NULL, },
+	enc_blocksize: 	AES_CBC_BLOCK_SIZE,
+	keyminlen: 	AES_KEY_MIN_LEN + 3,
+	keydeflen: 	AES_KEY_DEF_LEN + 3,
+	keymaxlen: 	AES_KEY_MAX_LEN + 3,
+};
+
+struct encrypt_desc algo_aes_gcm_8 =
+{
+	common: {
+	  name: "aes_gcm_8",
+	  officname: "aes_gcm_8",
+	  algo_type: 	IKE_ALG_ENCRYPT,
+	  algo_v2id:    IKEv2_ENCR_AES_GCM_8,
+	  algo_next: 	NULL, },
+	enc_blocksize: 	AES_CBC_BLOCK_SIZE,
+	keyminlen: 	AES_KEY_MIN_LEN + 3,
+	keydeflen: 	AES_KEY_DEF_LEN + 3,
+	keymaxlen: 	AES_KEY_MAX_LEN + 3,
+};
+
+struct encrypt_desc algo_aes_gcm_12 =
+{
+	common: {
+	  name: "aes_gcm_12",
+	  officname: "aes_gcm_12",
+	  algo_type: 	IKE_ALG_ENCRYPT,
+	  algo_v2id:    IKEv2_ENCR_AES_GCM_12,
+	  algo_next: 	NULL, },
+	enc_blocksize: 	AES_CBC_BLOCK_SIZE,
+	keyminlen: 	AES_KEY_MIN_LEN + 3,
+	keydeflen: 	AES_KEY_DEF_LEN + 3,
+	keymaxlen: 	AES_KEY_MAX_LEN + 3,
+};
+
+struct encrypt_desc algo_aes_gcm_16 =
+{
+	common: {
+	  name: "aes_gcm_16",
+	  officname: "aes_gcm_16",
+	  algo_type: 	IKE_ALG_ENCRYPT,
+	  algo_v2id:    IKEv2_ENCR_AES_GCM_16,
+	  algo_next: 	NULL, },
+	enc_blocksize: 	AES_CBC_BLOCK_SIZE,
+	keyminlen: 	AES_KEY_MIN_LEN + 3,
+	keydeflen: 	AES_KEY_DEF_LEN + 3,
+	keymaxlen: 	AES_KEY_MAX_LEN + 3,
+};
+
+static void
+linux_pfkey_add_aead(void)
+{
+	struct sadb_alg alg;
+
+	alg.sadb_alg_ivlen = 8;
+	alg.sadb_alg_minbits = 128;
+	alg.sadb_alg_maxbits = 256;
+
+	alg.sadb_alg_id = SADB_X_EALG_AES_GCM_ICV8;
+	kernel_alg_add(SADB_SATYPE_ESP, SADB_EXT_SUPPORTED_ENCRYPT, &alg);
+	alg.sadb_alg_id = SADB_X_EALG_AES_GCM_ICV12;
+	kernel_alg_add(SADB_SATYPE_ESP, SADB_EXT_SUPPORTED_ENCRYPT, &alg);
+	alg.sadb_alg_id = SADB_X_EALG_AES_GCM_ICV16;
+	kernel_alg_add(SADB_SATYPE_ESP, SADB_EXT_SUPPORTED_ENCRYPT, &alg);
+	alg.sadb_alg_id = SADB_X_EALG_AES_CCM_ICV8;
+	kernel_alg_add(SADB_SATYPE_ESP, SADB_EXT_SUPPORTED_ENCRYPT, &alg);
+	alg.sadb_alg_id = SADB_X_EALG_AES_CCM_ICV12;
+	kernel_alg_add(SADB_SATYPE_ESP, SADB_EXT_SUPPORTED_ENCRYPT, &alg);
+	alg.sadb_alg_id = SADB_X_EALG_AES_CCM_ICV16;
+	kernel_alg_add(SADB_SATYPE_ESP, SADB_EXT_SUPPORTED_ENCRYPT, &alg);
+
+	ike_alg_register_enc(&algo_aes_ccm_8);
+	ike_alg_register_enc(&algo_aes_ccm_12);
+	ike_alg_register_enc(&algo_aes_ccm_16);
+	ike_alg_register_enc(&algo_aes_gcm_8);
+	ike_alg_register_enc(&algo_aes_gcm_12);
+	ike_alg_register_enc(&algo_aes_gcm_16);
+}
+
 static void
 linux_pfkey_register_response(const struct sadb_msg *msg)
 {
@@ -752,6 +946,8 @@ linux_pfkey_register_response(const struct sadb_msg *msg)
 #ifdef KERNEL_ALG
 	    kernel_alg_register_pfkey(msg, msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN);
 #endif
+	    /* XXX Need to grab list from the kernel. */
+	    linux_pfkey_add_aead();
 	    break;
     case SADB_X_SATYPE_IPCOMP:
 	can_do_IPcomp = TRUE;
@@ -1154,7 +1350,7 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
     {
         inner_spi = st->st_ipcomp.attrs.spi;
         inner_proto = SA_COMP;
-        inner_satype = SADB_X_SATYPE_COMP;
+        inner_satype = K_SADB_X_SATYPE_COMP;
 
         i--;
         proto_info[i].proto = IPPROTO_COMP;
@@ -1174,7 +1370,7 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
 
         inner_spi = st->st_tunnel_out_spi;
         inner_proto = SA_IPIP;
-        inner_satype = SADB_X_SATYPE_IPIP;
+        inner_satype = K_SADB_X_SATYPE_IPIP;
 
         proto_info[i].encapsulation = ENCAPSULATION_MODE_TUNNEL;
         for (j = i + 1; proto_info[j].proto; j++)
@@ -1189,16 +1385,109 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
 }
 
 static bool
-netlink_shunt_eroute_fake(struct connection *c UNUSED
-                   , struct spd_route *sr UNUSED
+netlink_shunt_eroute(struct connection *c 
+                   , struct spd_route *sr 
                    , enum routing_t rt_kind
-                   , enum pluto_sadb_operations op UNUSED
+                   , enum pluto_sadb_operations op 
 		   , const char *opname)
 {
-	loglog(RC_COMMENT, "request to %s a %s policy with netkey kernel --- not yet implemented"
+	loglog(RC_COMMENT, "request to %s a %s policy with netkey kernel --- experimental"
 		, opname
 		, enum_name(&routing_story, rt_kind));
-	return TRUE;
+
+    /* We are constructing a special SAID for the eroute.
+     * The destination doesn't seem to matter, but the family does.
+     * The protocol is SA_INT -- mark this as shunt.
+     * The satype has no meaning, but is required for PF_KEY header!
+     * The SPI signifies the kind of shunt.
+     */
+    ipsec_spi_t spi = shunt_policy_spi(c, rt_kind == RT_ROUTED_PROSPECTIVE);
+
+    if (spi == 0)
+    {
+        /* we're supposed to end up with no eroute: rejig op and opname */
+        switch (op)
+        {
+        case ERO_REPLACE:
+            /* replace with nothing == delete */
+            op = ERO_DELETE;
+            opname = "delete";
+            break;
+        case ERO_ADD:
+            /* add nothing == do nothing */
+            return TRUE;
+        case ERO_DELETE:
+            /* delete remains delete */
+            break;
+
+	case ERO_ADD_INBOUND:
+		break;;
+
+	case ERO_DEL_INBOUND:
+		break;;
+
+        default:
+            bad_case(op);
+        }
+    }
+    if (sr->routing == RT_ROUTED_ECLIPSED && c->kind == CK_TEMPLATE)
+    {
+        /* We think that we have an eroute, but we don't.
+         * Adjust the request and account for eclipses.
+         */
+        passert(eclipsable(sr));
+        switch (op)
+        {
+        case ERO_REPLACE:
+            /* really an add */
+            op = ERO_ADD;
+            opname = "replace eclipsed";
+            eclipse_count--;
+            break;
+        case ERO_DELETE:
+            /* delete unnecessary: we don't actually have an eroute */
+            eclipse_count--;
+            return TRUE;
+        case ERO_ADD:
+        default:
+            bad_case(op);
+        }
+    }
+    else if (eclipse_count > 0 && op == ERO_DELETE && eclipsable(sr))
+    {
+        /* maybe we are uneclipsing something */
+        struct spd_route *esr;
+        struct connection *ue = eclipsed(c, &esr);
+
+        if (ue != NULL)
+        {
+            esr->routing = RT_ROUTED_PROSPECTIVE;
+            return netlink_shunt_eroute(ue, esr
+                                , RT_ROUTED_PROSPECTIVE, ERO_REPLACE, "restoring eclipsed");
+        }
+    }
+
+    {
+      const ip_address *peer = &sr->that.host_addr;
+      char buf2[256];
+      const struct af_info *fam = aftoinfo(addrtypeof(peer));
+      
+      if(fam == NULL) {
+	      fam=aftoinfo(AF_INET);
+      }
+      
+      snprintf(buf2, sizeof(buf2)
+	       , "eroute_connection %s", opname);
+
+      return netlink_raw_eroute(&sr->this.host_addr, &sr->this.client
+			      , fam->any
+			      , &sr->that.client
+			      , htonl(spi)
+			      , SA_INT
+			      , sr->this.protocol
+			      , K_SADB_X_SATYPE_INT
+			      , null_proto_info, 0, op, buf2);
+    }
 }
 
 static void
@@ -1431,6 +1720,7 @@ add_entry:
     }
 }
 
+
 const struct kernel_ops netkey_kernel_ops = {
     kern_name: "netkey",
     type: USE_NETKEY,
@@ -1450,14 +1740,13 @@ const struct kernel_ops netkey_kernel_ops = {
     grp_sa: NULL,
     get_spi: netlink_get_spi,
     docommand: do_command_linux,
+    process_ifaces: netlink_process_raw_ifaces,
 
     /* XXX these needed to be added */
-    shunt_eroute: netlink_shunt_eroute_fake,
+    shunt_eroute: netlink_shunt_eroute,
     sag_eroute: netlink_sag_eroute,   /* pfkey_sag_eroute, */
     eroute_idle: NULL,  /* pfkey_was_eroute_idle,*/
     set_debug: NULL,    /* pfkey_set_debug, */
     remove_orphaned_holds: NULL, /* pfkey_remove_orphaned_holds,*/
-
-    process_ifaces: netlink_process_raw_ifaces,
 };
 #endif /* linux && NETKEY_SUPPORT */
