@@ -280,6 +280,7 @@ free_dead_ifaces(void)
 	struct iface_port **pp;
 
 	release_dead_interfaces();
+	delete_states_dead_interfaces();
 	for (pp = &interfaces; (p = *pp) != NULL; )
 	{
 	    if (p->change == IFN_DELETE)
@@ -387,13 +388,12 @@ create_socket(struct raw_iface *ifp, const char *v_name, int port)
 	struct sadb_x_policy policy;
 	int level, opt;
 
+	memset(&policy, 0, sizeof(struct sadb_x_policy));
 	policy.sadb_x_policy_len = sizeof(policy) / IPSEC_PFKEYv2_ALIGN;
 	policy.sadb_x_policy_exttype = SADB_X_EXT_POLICY;
 	policy.sadb_x_policy_type = IPSEC_POLICY_BYPASS;
 	policy.sadb_x_policy_dir = IPSEC_DIR_INBOUND;
-	policy.sadb_x_policy_reserved = 0;
 	policy.sadb_x_policy_id = 0;
-	policy.sadb_x_policy_reserved2 = 0;
 
 	if (addrtypeof(&ifp->addr) == AF_INET6)
 	{
@@ -864,9 +864,7 @@ check_msg_errqueue(const struct iface_port *ifp, short interest)
 	    struct sockaddr_in6 sa_in6;
 	} from;
 
-	int from_len = sizeof(from);
-
-	int packet_len;
+	ssize_t packet_len;
 
 	struct msghdr emh;
 	struct iovec eiov;
@@ -883,7 +881,6 @@ check_msg_errqueue(const struct iface_port *ifp, short interest)
 	struct state *sender = NULL;
 
 	zero(&from.sa);
-	from_len = sizeof(from);
 
 	emh.msg_name = &from.sa;	/* ??? filled in? */
 	emh.msg_namelen = sizeof(from);
