@@ -562,6 +562,30 @@ static bool same_phase1_sa_relations(struct state *this
 		&& this->st_clonedfrom == parent_sa));
 }
 
+/*
+ * Delete all states that have somehow not ben deleted yet
+ * but using interfaces that are going down
+ */
+ 
+void delete_states_dead_interfaces(void)
+{
+    struct state *st = NULL;
+    int i;
+
+    for (i = 0; st == NULL && i < STATE_TABLE_SIZE; i++)
+	for (st = statetable[i]; st != NULL;){
+	    struct state *this = st;
+	    st = st->st_hashchain_next;	/* before this is deleted */
+	    if (this->st_interface && this->st_interface->change == IFN_DELETE )
+	    {
+		openswan_log("deleting lasting state #%lu on interface (%s) which is shutting down",
+			this->st_serialno,
+			this->st_interface->ip_dev->id_vname);
+		delete_state(this);
+	    }
+	}
+}
+
 /*
  * delete all states that were created for a given connection.
  * if relations == TRUE, then also delete states that share
@@ -1235,7 +1259,7 @@ void fmt_state(struct state *st, time_t n
 	/* XXX - mcr last used is really an attribute of the connection */
 	lastused[0] = '\0';
 	if (c->spd.eroute_owner == st->st_serialno
-	&& st->st_outbound_count != 0)
+	    && st->st_outbound_count != 0)
 	{
 	    snprintf(lastused, sizeof(lastused)
 		, " used %lus ago;"
@@ -1260,19 +1284,20 @@ void fmt_state(struct state *st, time_t n
 	}
 #ifdef KLIPS
 	if (st->st_ah.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
-	|| st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
-	|| st->st_ipcomp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
+	    || st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
+	    || st->st_ipcomp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
 	{
 	    add_said(&c->spd.that.host_addr, st->st_tunnel_out_spi, SA_IPIP);
 	    add_said(&c->spd.this.host_addr, st->st_tunnel_in_spi, SA_IPIP);
 	}
 #endif
 	snprintf(state_buf2, state_buf2_len
-	    , "#%lu: \"%s\"%s%s%s"
+	    , "#%lu: \"%s\"%s%s%s ref=%lu refhim=%lu"
 	    , st->st_serialno
 	    , c->name, inst
 	    , lastused
-	    , buf);
+	    , buf
+		 , (unsigned long)st->st_ref, (unsigned long)st->st_refhim);
 
 #	undef add_said
     }
