@@ -190,8 +190,9 @@ struct state
     ipsec_spi_t        st_tunnel_in_spi;          /* KLUDGE */
     ipsec_spi_t        st_tunnel_out_spi;         /* KLUDGE */
 
-    IPsecSAref_t       ref;	   /* our kernel name for our incoming SA */
-    IPsecSAref_t       refhim;     /* our kernel name for our outgoing SA */
+    IPsecSAref_t       st_ref;	   /* our kernel name for our incoming SA */
+    IPsecSAref_t       st_refhim;     /* our kernel name for our outgoing SA */
+    bool               st_outbound_done;         /* if true, then outgoing SA already installed */
 
     const struct oakley_group_desc *st_pfs_group; /*group for Phase 2 PFS */
 
@@ -423,6 +424,7 @@ extern void set_state_ike_endpoints(struct state *st
 				    , struct connection *c);
 
 extern void delete_cryptographic_continuation(struct state *st);
+extern void delete_states_dead_interfaces(void);
 
 #endif /* _STATE_H */
 
