@@ -253,6 +253,8 @@ static struct vid_struct _vid_tab[] = {
 		"\xaf\xca\xd7\x13\x68\xa1\xf1\xc9\x6b\x86\x96\xfc\x77\x57\x01\x00",
 		16 },
 
+	{ VID_MISC_IKEv2, VID_STRING|VID_KEEP, "IKEv2", "CAN-IKEv2", NULL, 0},
+
 	/**
 	 * Netscreen:
 	 * 4865617274426561745f4e6f74696679386b0100  (HeartBeat_Notify + 386b0100)
@@ -393,7 +395,7 @@ void init_vendorid(void)
  * @param st State Structure (Hopefully initialized)
  * @return void
  */
-static void handle_known_vendorid (struct msg_digest *md UNUSED
+static void handle_known_vendorid (struct msg_digest *md 
 				   , const char *vidstr
 				   , size_t len
 				   , struct vid_struct *vid
@@ -461,6 +463,11 @@ static void handle_known_vendorid (struct msg_digest *md UNUSED
 	    vid_usefull = 1;
             break;
 
+	case VID_MISC_IKEv2:
+	    md->ikev2 = TRUE;
+	    vid_usefull = 1;
+	    break;
+
 /* We only need these when dealing with XAUTH */
 #ifdef XAUTH
 	case VID_SSH_SENTINEL_1_4_1:
