@@ -27,6 +27,7 @@ char showpolicy_version[] = "RCSID $Id: showpolicy.c,v 1.5 2004/04/04 01:50:56 k
 #include "openswan/ipsec_policy.h"
 #include "sysdep.h"
 #include "socketwrapper.h"
+#include "rcvinfo.h"
 
 char *program_name;
 
@@ -62,19 +63,19 @@ static const struct option long_opts[] = {
     { "cgi",         no_argument, NULL, 'g' },
     { "textual",     no_argument, NULL, 't' },
     { "plaintext",   required_argument, NULL, 'c' },
+    { "maxpacket",   required_argument, NULL, 'N' },
     { "vpntext",     required_argument, NULL, 'v' },
     { "privacytext", required_argument, NULL, 'p' },
     { "dnssectext",  required_argument, NULL, 's' },
     { 0,0,0,0 }
 };
 
-#ifndef IP_IPSEC_RECVREF
-#define IP_IPSEC_RECVREF 18
-#endif
+int maxpacketcount=0;
 
 int open_udp_sock(unsigned short port)
 {
 	struct sockaddr_in s;
+	int one;
 	int fd;
 
 	fd = safe_socket(PF_INET, SOCK_DGRAM, 0);
@@ -85,7 +86,7 @@ int open_udp_sock(unsigned short port)
 
 	memset(&s, 0, sizeof(struct sockaddr_in));
 	s.sin_family = AF_INET;
-	s.sin_port   = port;
+	s.sin_port   = htons(port);
 	s.sin_addr.s_addr = INADDR_ANY;
 
 	if(bind(fd, (struct sockaddr *)&s, sizeof(struct sockaddr_in))==-1) {
@@ -93,16 +94,11 @@ int open_udp_sock(unsigned short port)
 		exit(11);
 	}
 
-#ifdef SAREF_SUPPORTED
-	{
-	  int one;
-	  one = 1;
-	  if(setsockopt(fd, SOL_IP, IP_IPSEC_RECVREF, &one, sizeof(one)) != 0) {
-		  perror("setsockopt recvref");
-		  exit(12);
-	  }
+	one = 1;
+	if(setsockopt(fd, SOL_IP, IP_IPSEC_REFINFO, &one, sizeof(one)) != 0) {
+		perror("setsockopt recvref");
+		exit(12);
 	}
-#endif
 	return fd;
 }
 
@@ -116,13 +112,15 @@ int udp_recv_loop(int udpsock)
 	struct sockaddr_in from, to;
 	int fromlen, tolen;
 	struct msghdr msgh;
-	struct cmsghdr *cmsg;
 	struct iovec iov;
 	char cbuf[256];
-	int err;
+	int  readlen, err;
 	char buf[512];
+	int packetcount =0;
 	
 	do {
+		unsigned int pktref[1];
+
 		memset(&from, 0, sizeof(from));
 		memset(&to,   0, sizeof(to));
 
@@ -141,31 +139,87 @@ int udp_recv_loop(int udpsock)
 		msgh.msg_flags = 0;
 
 		/* Receive one packet. */
-		if ((err = recvmsg(udpsock, &msgh, 0)) < 0) {
-			return err;
+		if ((readlen = recvmsg(udpsock, &msgh, 0)) < 0) {
+			return readlen;
 		}
 		
-		/* Process auxiliary received data in msgh */
-		for (cmsg = CMSG_FIRSTHDR(&msgh);
-		     cmsg != NULL;
-		     cmsg = CMSG_NXTHDR(&msgh,cmsg)) {
-			printf("cmsg_level: %d type: %d\n",
-			       cmsg->cmsg_level, cmsg->cmsg_type);
-			
-			if (cmsg->cmsg_level == IPPROTO_IP
-			    && cmsg->cmsg_type == IP_IPSEC_RECVREF) {
-				unsigned int *refp;
+		printf("got message of length: %d\n", readlen);
+		
+		{
+			struct cmsghdr *cmsg;
+			/* Process auxiliary received data in msgh */
+			for (cmsg = CMSG_FIRSTHDR(&msgh);
+			     cmsg != NULL;
+			     cmsg = CMSG_NXTHDR(&msgh,cmsg)) {
+				printf("cmsg_level: %d type: %d\n",
+				       cmsg->cmsg_level, cmsg->cmsg_type);
+				
+				if (cmsg->cmsg_level == IPPROTO_IP
+				    && cmsg->cmsg_type == IP_IPSEC_REFINFO) {
+					unsigned int *refp;
+					
+					refp = (unsigned int *)CMSG_DATA(cmsg);
+					pktref[0]=refp[0];
+					pktref[1]=refp[1];
 
-				refp = (unsigned int *)CMSG_DATA(cmsg);
-				printf("     refp=%08x (%u)\n", *refp, *refp);
+					printf("     ref=%08x (%u) him=%08x (%u)\n",
+					       pktref[0], pktref[0],
+					       pktref[1], pktref[1]);
+				}
 			}
 		}
-	} while(err != -1);
+
+		/*
+		 * OKAY, now send the packet back again.
+		 */
+		memset(&msgh, 0, sizeof(struct msghdr));
+
+		{
+			struct cmsghdr *cmsg;
+			char cbuf[CMSG_SPACE(sizeof (unsigned int))];
+			unsigned int *refp;
+		
+			msgh.msg_control = cbuf;
+			msgh.msg_controllen = sizeof(cbuf);
+
+			cmsg = CMSG_FIRSTHDR(&msgh);
+			cmsg->cmsg_level = SOL_IP;
+			cmsg->cmsg_type  = IP_IPSEC_REFINFO;
+			cmsg->cmsg_len   = CMSG_LEN(sizeof(unsigned int));
+			
+			if(pktref[1]) {
+				/* why are we setting refp, we don't use it? */
+				printf("     sending with saref=%d\n", pktref[1]);
+				refp = (unsigned int *)CMSG_DATA(cmsg);
+				*refp = pktref[1];
+			}
+			
+			iov.iov_base = buf;
+			iov.iov_len  = readlen;
+
+			/* return packet from whence it came */
+			msgh.msg_name = &from;
+			msgh.msg_namelen = fromlen;
+
+			msgh.msg_iov  = &iov;
+			msgh.msg_iovlen = 1;
+			msgh.msg_flags = 0;
+		}
+
+		/* Receive one packet. */
+		if ((err = sendmsg(udpsock, &msgh, 0)) < 0) {
+			perror("sendmsg");
+			err = 0;
+		}
+		
+		printf("sent message of length: %d\n", err);
+		packetcount++;
+
+	} while(err != -1 && (maxpacketcount==0 || packetcount<maxpacketcount));
 
 	return 0;
 }
 	
-	
 
 void dump_policyreply(struct ipsec_policy_cmd_query *q)
 {
@@ -271,6 +325,14 @@ int main(int argc, char *argv[])
 	    fd = open_tcp_sock(port);
 	    break;
       
+    case 'N':
+	    maxpacketcount = strtol(optarg, &foo, 0);
+	    if(*foo != '\0') {
+		    fprintf(stderr, "invalid packetcount number: %s\n", optarg);
+		    help();
+	    }
+	    break;
+      
     case 'P':
       lookup_style = 'P';
       break;
@@ -301,9 +363,11 @@ int main(int argc, char *argv[])
     }
   }
 	
-  if((ret = ipsec_policy_init()) != NULL) {
-    perror(ret);
-    exit(2);
+  if(lookup_style != 'P') {
+	  if((ret = ipsec_policy_init()) != NULL) {
+		  perror(ret);
+		  exit(2);
+	  }
   }
 
   switch(lookup_style) {
