@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char spigrp_c_version[] = "RCSID $Id: spigrp.c,v 1.50.2.1 2005-08-18 14:04:51 ken Exp $";
+char spigrp_c_version[] = "RCSID $Id: spigrp.c,v 1.51 2005/08/18 14:04:39 ken Exp $";
 
 
 #include <sys/types.h>
@@ -41,15 +41,16 @@ char spigrp_c_version[] = "RCSID $Id: spigrp.c,v 1.50.2.1 2005-08-18 14:04:51 ke
 #endif
 
 #include <signal.h>
-#include <pfkeyv2.h>
-#include <pfkey.h>
+#include <openswan/pfkeyv2.h>
+#include <openswan/pfkey.h>
+#include "pfkey_help.h"
 
 #include "openswan/radij.h"
 #include "openswan/ipsec_encap.h"
 #include "openswan/ipsec_ah.h"
 
 
-char *program_name;
+char *progname;
 
 int pfkey_sock;
 fd_set pfkey_socks;
@@ -78,6 +79,8 @@ usage(char *s)
 }
 
 	
+int debug = 0;
+
 int
 main(int argc, char **argv)
 {
@@ -87,19 +90,18 @@ main(int argc, char **argv)
 
 	const char* error_s = NULL;
 	char ipaddr_txt[ADDRTOT_BUF];
-	int debug = 0;
 	int j;
 	struct said_af said_af_array[4];
 
 	int error = 0;
 
-	struct sadb_ext *extensions[SADB_EXT_MAX + 1];
+	struct sadb_ext *extensions[K_SADB_EXT_MAX + 1];
 	struct sadb_msg *pfkey_msg;
 #if 0
 	ip_address pfkey_address_s_ska;
 #endif
 	
-	program_name = argv[0];
+	progname = argv[0];
 	for(i = 0; i < 4; i++) {
 		memset(&said_af_array[i], 0, sizeof(struct said_af));
 	}
@@ -122,20 +124,20 @@ main(int argc, char **argv)
 
         if(argc > 1 && strcmp(argv[1], "--label") == 0) {
 		if(argc > 2) {
-			program_name = malloc(strlen(argv[0])
+			progname = malloc(strlen(argv[0])
 					      + 10 /* update this when changing the sprintf() */
 					      + strlen(argv[2]));
-			sprintf(program_name, "%s --label %s",
+			sprintf(progname, "%s --label %s",
 				argv[0],
 				argv[2]);
 			if(debug) {
-				fprintf(stdout, "using \"%s\" as a label.\n", program_name);
+				fprintf(stdout, "using \"%s\" as a label.\n", progname);
 			}
 			argv += 2;
 			argc -= 2;
 		} else {
 			fprintf(stderr, "%s: --label option requires an argument.\n",
-				program_name);
+				progname);
 			exit(1);
 		}
         }
@@ -157,7 +159,7 @@ main(int argc, char **argv)
 		if(debug) {
 			fprintf(stdout, "\"--help\" option requested.\n");
 		}
-                usage(program_name);
+                usage(progname);
                 exit(1);
         }
 
@@ -169,7 +171,7 @@ main(int argc, char **argv)
 		if(debug) {
 			fprintf(stdout, "\"--version\" option requested.\n");
 		}
-                fprintf(stderr, "%s, %s\n", program_name, spigrp_c_version);
+                fprintf(stderr, "%s, %s\n", progname, spigrp_c_version);
                 exit(1);
         }
 
@@ -191,14 +193,14 @@ main(int argc, char **argv)
 	if(said_opt) {
 		if (argc < 3 /*|| argc > 5*/) {
 			fprintf(stderr, "expecting 3 or more args with --said, got %d.\n", argc);
-			usage(program_name);
+			usage(progname);
                 	exit(1);
 		}
 		nspis = argc - 2;
 	} else {
 		if ((argc < 5) || (argc > 17) || ((argc % 4) != 1)) {
 			fprintf(stderr, "expecting 5 or more args without --said, got %d.\n", argc);
-			usage(program_name);
+			usage(progname);
                 	exit(1);
 		}
 		nspis = argc / 4;
@@ -217,7 +219,7 @@ main(int argc, char **argv)
 			error_s = ttosa((const char *)argv[i+2], 0, (ip_said*)&(said_af_array[i].said));
 			if(error_s != NULL) {
 				fprintf(stderr, "%s: Error, %s converting --sa argument:%s\n",
-					program_name, error_s, argv[i+2]);
+					progname, error_s, argv[i+2]);
 				exit (1);
 			}
 			said_af_array[i].af = addrtypeof(&(said_af_array[i].said.dst));
@@ -240,13 +242,13 @@ main(int argc, char **argv)
 			}
 			if(said_af_array[i].said.proto == 0) {
 				fprintf(stderr, "%s: Badly formed proto: %s\n",
-					program_name, argv[i*4+4]);
+					progname, argv[i*4+4]);
 				exit(1);
 			}
 			said_af_array[i].said.spi = htonl(strtoul(argv[i*4+3], &endptr, 0));
 			if(!(endptr == argv[i*4+3] + strlen(argv[i*4+3]))) {
 				fprintf(stderr, "%s: Badly formed spi: %s\n",
-					program_name, argv[i*4+3]);
+					progname, argv[i*4+3]);
 				exit(1);
 			}
 			if(!strcmp(argv[i*4+1], "inet")) {
@@ -257,13 +259,13 @@ main(int argc, char **argv)
 			}
 			if((said_af_array[i].af != AF_INET) && (said_af_array[i].af != AF_INET6)) {
 				fprintf(stderr, "%s: Address family %s not supported\n",
-					program_name, argv[i*4+1]);
+					progname, argv[i*4+1]);
 				exit(1);
 			}
 			error_s = ttoaddr(argv[i*4+2], 0, said_af_array[i].af, &(said_af_array[i].said.dst));
 			if(error_s != NULL) {
 				fprintf(stderr, "%s: Error, %s converting %dth address argument:%s\n",
-					program_name, error_s, i, argv[i*4+2]);
+					progname, error_s, i, argv[i*4+2]);
 				exit (1);
 			}
 		}
@@ -281,52 +283,9 @@ main(int argc, char **argv)
 		fprintf(stdout, "Opening pfkey socket.\n");
 	}
 
-	if((pfkey_sock = socket(PF_KEY, SOCK_RAW, PF_KEY_V2) ) < 0) {
-		fprintf(stderr, "%s: Trouble opening PF_KEY family socket with error: ",
-			program_name);
-		switch(errno) {
-		case ENOENT:
-			fprintf(stderr, "ipsec# device does not exist.  See Openswan installation procedure.\n");
-			break;
-		case EACCES:
-			fprintf(stderr, "access denied.  ");
-			if(getuid() == 0) {
-				fprintf(stderr, "Check permissions, they should be set to 600.\n");
-			} else {
-				fprintf(stderr, "You must be root to open this file.\n");
-			}
-			break;
-		case EUNATCH:
-			fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
-			break;
-		case ENODEV:
-			fprintf(stderr, "KLIPS not loaded or enabled.\n");
-			break;
-		case EBUSY:
-			fprintf(stderr, "KLIPS is busy.  Most likely a serious internal error occured in a previous command.  Please report as much detail as possible to development team.\n");
-			break;
-		case EINVAL:
-			fprintf(stderr, "Invalid argument, KLIPS not loaded or check kernel log messages for specifics.\n");
-			break;
-		case ENOBUFS:
-			fprintf(stderr, "No kernel memory to allocate SA.\n");
-			break;
-		case ESOCKTNOSUPPORT:
-			fprintf(stderr, "Algorithm support not available in the kernel.  Please compile in support.\n");
-			break;
-		case EEXIST:
-			fprintf(stderr, "SA already in use.  Delete old one first.\n");
-			break;
-		case ENXIO:
-			fprintf(stderr, "SA does not exist.  Cannot delete.\n");
-			break;
-		case EAFNOSUPPORT:
-			fprintf(stderr, "KLIPS not loaded or enabled.\n");
-			break;
-		default:
-			fprintf(stderr, "Unknown file open error %d.  Please report as much detail as possible to development team.\n", errno);
-		}
-		exit(1);
+	pfkey_sock = pfkey_open_sock_with_error();
+	if(pfkey_sock < 0) {
+	    exit(1);
 	}
 
 	for(i = 0; i < (((nspis - 1) < 2) ? 1 : (nspis - 1)); i++) {
@@ -344,13 +303,13 @@ main(int argc, char **argv)
 			/* It needs <base, SA, SA2, address(D,D2) > minimum. */
 			if(!j) {
 				if((error = pfkey_msg_hdr_build(&extensions[0],
-								SADB_X_GRPSA,
+								K_SADB_X_GRPSA,
 								proto2satype(said_af_array[i].said.proto),
 								0,
 								++pfkey_seq,
 								getpid()))) {
 					fprintf(stderr, "%s: Trouble building message header, error=%d.\n",
-						program_name, error);
+						progname, error);
 					pfkey_extensions_free(extensions);
 					exit(1);
 				}
@@ -362,18 +321,18 @@ main(int argc, char **argv)
 						);
 				}
 
-				if((error = pfkey_x_satype_build(&extensions[SADB_X_EXT_SATYPE2],
+				if((error = pfkey_x_satype_build(&extensions[K_SADB_X_EXT_SATYPE2],
 								 proto2satype(said_af_array[i+j].said.proto)
 					))) {
 					fprintf(stderr, "%s: Trouble building message header, error=%d.\n",
-						program_name, error);
+						progname, error);
 					pfkey_extensions_free(extensions);
 					exit(1);
 				}
 			}
 
-			if((error = pfkey_sa_build(&extensions[!j ? SADB_EXT_SA : SADB_X_EXT_SA2],
-						   !j ? SADB_EXT_SA : SADB_X_EXT_SA2,
+			if((error = pfkey_sa_build(&extensions[!j ? K_SADB_EXT_SA : K_SADB_X_EXT_SA2],
+						   !j ? K_SADB_EXT_SA : K_SADB_X_EXT_SA2,
 						   said_af_array[i+j].said.spi, /* in network order */
 						   0,
 						   0,
@@ -381,7 +340,7 @@ main(int argc, char **argv)
 						   0,
 						   0))) {
 				fprintf(stderr, "%s: Trouble building sa extension, error=%d.\n",
-					program_name, error);
+					progname, error);
 				pfkey_extensions_free(extensions);
 				exit(1);
 			}
@@ -389,14 +348,14 @@ main(int argc, char **argv)
 #if 0
 			if(!j) {
 				anyaddr(said_af_array[i].af, &pfkey_address_s_ska); /* Is the address family correct ?? */
-				if((error = pfkey_address_build(&extensions[SADB_EXT_ADDRESS_SRC],
-								SADB_EXT_ADDRESS_SRC,
+				if((error = pfkey_address_build(&extensions[K_SADB_EXT_ADDRESS_SRC],
+								K_SADB_EXT_ADDRESS_SRC,
 								0,
 								0,
 								sockaddrof(&pfkey_address_s_ska)))) {
 					addrtot(&pfkey_address_s_ska, 0, ipaddr_txt, sizeof(ipaddr_txt));
 					fprintf(stderr, "%s: Trouble building address_s extension (%s), error=%d.\n",
-						program_name, ipaddr_txt, error);
+						progname, ipaddr_txt, error);
 					pfkey_extensions_free(extensions);
 					exit(1);
 				}
@@ -410,7 +369,7 @@ main(int argc, char **argv)
 				addrtot(&said_af_array[i+j].said.dst,
 					0, ipaddr_txt, sizeof(ipaddr_txt));
 				fprintf(stderr, "%s: Trouble building address_d extension (%s), error=%d.\n",
-					program_name, ipaddr_txt, error);
+					progname, ipaddr_txt, error);
 				pfkey_extensions_free(extensions);
 				exit(1);
 			}
@@ -419,7 +378,7 @@ main(int argc, char **argv)
 
 		if((error = pfkey_msg_build(&pfkey_msg, extensions, EXT_BITS_IN))) {
 			fprintf(stderr, "%s: Trouble building pfkey message, error=%d.\n",
-				program_name, error);
+				progname, error);
 			pfkey_extensions_free(extensions);
 			pfkey_msg_free(&pfkey_msg);
 			exit(1);
@@ -430,57 +389,12 @@ main(int argc, char **argv)
 				  pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN)) !=
 		   (ssize_t)(pfkey_msg->sadb_msg_len * IPSEC_PFKEYv2_ALIGN)) {
 			fprintf(stderr, "%s: pfkey write failed, returning %d with errno=%d.\n",
-				program_name, error, errno);
+				progname, error, errno);
 			pfkey_extensions_free(extensions);
 			pfkey_msg_free(&pfkey_msg);
-			switch(errno) {
-			case EACCES:
-				fprintf(stderr, "access denied.  ");
-				if(getuid() == 0) {
-					fprintf(stderr, "Check permissions.  Should be 600.\n");
-				} else {
-					fprintf(stderr, "You must be root to open this file.\n");
-				}
-				break;
-			case EUNATCH:
-				fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
-				break;
-			case EBUSY:
-				fprintf(stderr, "KLIPS is busy.  Most likely a serious internal error occured in a previous command.  Please report as much detail as possible to development team.\n");
-				break;
-			case EINVAL:
-				fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
-				break;
-			case ENODEV:
-				fprintf(stderr, "KLIPS not loaded or enabled.\n");
-				fprintf(stderr, "No device?!?\n");
-				break;
-			case ENOBUFS:
-				fprintf(stderr, "No kernel memory to allocate SA.\n");
-				break;
-			case ESOCKTNOSUPPORT:
-				fprintf(stderr, "Algorithm support not available in the kernel.  Please compile in support.\n");
-				break;
-			case EEXIST:
-				fprintf(stderr, "SA already in use.  Delete old one first.\n");
-				break;
-			case ENOENT:
-				fprintf(stderr, "device does not exist.  See FreeS/WAN installation procedure.\n");
-				break;
-			case ENXIO:
-				fprintf(stderr, "SA does not exist.  Cannot delete.\n");
-				break;
-			case ENOSPC:
-				fprintf(stderr, "no room in kernel SAref table.  Cannot process request.\n");
-				break;
-			case ESPIPE:
-				fprintf(stderr, "kernel SAref table internal error.  Cannot process request.\n");
-				break;
-			default:
-				fprintf(stderr, "Unknown socket write error %d.  Please report as much detail as possible to development team.\n", errno);
-			}
-			exit(1);
+			pfkey_write_error(error, errno);
 		}
+
 		if(pfkey_msg) {
 			pfkey_extensions_free(extensions);
 			pfkey_msg_free(&pfkey_msg);
@@ -490,9 +404,15 @@ main(int argc, char **argv)
 	(void) close(pfkey_sock);  /* close the socket */
 	exit(0);
 }
+
+void exit_tool(int x)
+{
+	exit(x);
+}
+
 /*
  * $Log: spigrp.c,v $
- * Revision 1.50.2.1  2005-08-18 14:04:51  ken
+ * Revision 1.51  2005/08/18 14:04:39  ken
  * Patch from mt@suse.de to avoid GCC warnings with system() calls
  *
  * Revision 1.50  2005/07/08 02:56:38  paul
