@@ -19,29 +19,12 @@ extern void *malloc(unsigned int size);
 #include "openswan/ipsec_sa.h"
 #include "openswan/ipsec_policy.h"
 #include "openswan/ipsec_proto.h"
+#include "openswan/ipsec_sysctl.h"
 #include "openswan/pfkeyv2.h"
 #include "openswan/pfkey.h"
 
 #include "talloc.h"
 
-int debug_tunnel;
-int debug_eroute;
-int debug_spi;
-int debug_radij;
-int debug_pfkey;
-int debug_ah;
-int debug_esp;
-int debug_netlink;
-int debug_rcv;
-int debug_xmit;
-extern int sysctl_ipsec_debug_verbose;
-int sysctl_ipsec_debug_ipcomp;
-extern int sysctl_ipsec_icmp;
-extern int sysctl_ipsec_tos;
-extern int debug_rcv;
-int sysctl_ip_default_ttl = 64;
-int sysctl_ipsec_inbound_policy_check = 1;
-
 struct prng ipsec_prng;
 spinlock_t eroute_lock = SPIN_LOCK_UNLOCKED;
 
@@ -189,6 +172,7 @@ int main(char *argv[], int argc)
 	       0x49, 0x4a, 0x4a, 0x4c, 0x4c, 0x4f, 0x4f, 0x51,
 	       0x51, 0x52, 0x52, 0x54, 0x54, 0x57, 0x57, 0x58};
 
+  debug_xform = 1;
   init_kmalloc();
   debug_tunnel=0xffffffff;
   debug_xmit=0xffffffff;
@@ -201,6 +185,8 @@ int main(char *argv[], int argc)
     sa1 = ipsec_sa_alloc(&error);
     assert(error == 0);
 
+    ipsec_sa_intern(sa1);
+
     sa1->ips_seq = 1;
     sa1->ips_pid = 10;
     
@@ -219,6 +205,8 @@ int main(char *argv[], int argc)
     sa = ipsec_sa_alloc(&error);
     assert(error == 0);
 
+    ipsec_sa_intern(sa);
+
     sa->ips_said.spi = htonl(0x12345678);
     sa->ips_said.proto = IPPROTO_ESP;
     sa->ips_said.dst.u.v4.sin_addr.s_addr = htonl(0xc001022D);
