@@ -1,3 +1,4 @@
+: ==== start ====
 route delete -net 192.0.2.0 netmask 255.255.255.0
 route delete -net default
 route add -net default gw 192.1.2.23
@@ -6,7 +7,7 @@ ipsec setup start
 
 /testing/pluto/bin/wait-until-pluto-started
 
-# ipsec manual --show --up westnet-east-pass
-ipsec manual --up westnet-east-pass
+# ipsec auto --show --up westnet-east-pass
+ipsec auto --up westnet-east-pass
 
 # : hello
